/* global BoltsSliderCaptions, BoltsVars */

// config
require.config( {
	paths: {
		jquery:     'assets/js/fix.jquery',
		underscore: 'assets/js/fix.underscore',
		util:       'bower_components/bootstrap/dist/js/umd/util',
		alert:      'bower_components/bootstrap/dist/js/umd/alert',
		button:     'bower_components/bootstrap/dist/js/umd/button',
		carousel:   'bower_components/bootstrap/dist/js/umd/carousel',
		collapse:   'bower_components/bootstrap/dist/js/umd/collapse',
		dropdown:   'bower_components/bootstrap/dist/js/umd/dropdown',
		modal:      'bower_components/bootstrap/dist/js/umd/modal',
		scrollspy:  'bower_components/bootstrap/dist/js/umd/scrollspy',
		tab:        'bower_components/bootstrap/dist/js/umd/tab',
		tooltip:    'bower_components/bootstrap/dist/js/umd/tooltip',
		popover:    'bower_components/bootstrap/dist/js/umd/popover',
		stampit:    'assets/js/vendor/stampit',
	}
} );

require.config( {
	baseUrl: BoltsVars.pathToTheme
} );

require( [
		'jquery',
		'underscore',
		'assets/js/utils/isElementInView',
		'assets/js/utils/objectFitFallback',
		'assets/js/WidgetLine',
		'assets/js/portfolio-grid-filter/gridFilter',
		'assets/js/portfolio-grid-filter/sliderFilter',
		'assets/js/utils/easeInOutQuad',
		'vendor/proteusthemes/proteuswidgets/assets/js/NumberCounter',
		'vendor/proteusthemes/sticky-menu/assets/js/sticky-menu',
		'assets/js/TouchDropdown',
		'carousel',
		'collapse',
		'tab',
], function ( $, _, isElementInView, objectFitFallback, WidgetLine, gridFilter, sliderFilter, easeInOutQuad, NumberCounter ) {
	'use strict';

	/**
	 * Footer widgets fix
	 */
	$( '.col-lg-__col-num__' ).removeClass( 'col-lg-__col-num__' ).addClass( 'col-lg-3' );

	/**
	 * Number Counter Widget JS code
	 */
	// Get all number counter widgets
	var $counterWidgets = $( '.number-counters' );

	if ( $counterWidgets.length ) {

		// jQuery easing function: easeInOutQuad, for use in NumberCounter
		easeInOutQuad();

		$counterWidgets.each( function () {
			new NumberCounter( $( this ) );
		} );
	}

	/**
	 * Widget lines, the length is relative to the amount of text
	 */
	$( '.widget-title--big .widget-title' ).each( function () {
		new WidgetLine( $( this ) );
	} );

	/**
	 * Animate the scroll, when back to top is clicked
	 */
	(function backToTop() {
		$('.footer-middle__back-to-top').click(function (ev) {
			ev.preventDefault();

			$('body, html').animate({
				scrollTop: 0
			}, 700);
		});
	})();

	/**
	 * Main slider controls
	 */
	(function slider () {
		var $slider = $('.js-jumbotron-slider'),
			$captions = {
				container: $('.js-slider-captions'),
				title:     $('.js-slider-captions__title'),
				text:      $('.js-slider-captions__text'),
			},
			transitionClass = 'is-in-transition',
			transitionTime = 600; // in ms

		// Captions for the main slider - change them in the out-of-the-bounds element
		if ( $slider.length && 'object' === typeof BoltsSliderCaptions ) {
			$slider.on('slide.bs.carousel', function (ev) {
				var nextIndex = $(ev.relatedTarget).index();

				$captions.container.addClass(transitionClass);

				setTimeout(_.bind(function () {
					$captions.title.text(BoltsSliderCaptions[nextIndex].title);
					$captions.text.html(BoltsSliderCaptions[nextIndex].text);
				}, this), (transitionTime / 2));
			});

			$slider.on('slid.bs.carousel', function () {
				$captions.container.removeClass(transitionClass);
			});
		}

		// Pause the carousel if it's not visible
		if ($slider.length) {
			$(document).on('scroll', _.throttle(function () {
				if (isElementInView($slider)) {
					$slider.carousel('cycle');
				}
				else {
					$slider.carousel('pause');
				}
			}, 1000, {leading: false}));
		}
	})();

	/**
	 * Portfolio grid filtering
	 */
	$('.portfolio-grid').each(function () {
		var hash = window.location.hash,
			portfolioGrid;

		if ('slider' === $(this).data('type')) {
			portfolioGrid = sliderFilter({
				$container: $(this),
			});
		}
		else {
			portfolioGrid = gridFilter({
				$container: $(this),
			});
		}

		// Getting on visit or if "All" nav button is disabled.
		if ( new RegExp('^#' + portfolioGrid.hashPrefix).test(hash) ) {
			$(this).find('a[href="' + hash.replace(portfolioGrid.hashPrefix, '') + '"]').trigger('click');
		}
		else if ( $(this).find('.portfolio-grid__nav-item').first().hasClass('is-disabled') ) {

			// Trigger click for the second nav grid item.
			$(this).find('.portfolio-grid__nav-item:nth-child(2)').children('.portfolio-grid__nav-link').trigger('click');
		}

		// Recalculate the mobile nav height, if the "All" nav button is disabled. Fix for both cases above.
		if ( ! portfolioGrid.isDesktopLayout() && $(this).find('.portfolio-grid__nav-item').first().hasClass('is-disabled') ) {
			portfolioGrid.initNavHolderHeight();
		}
	});

	/**
	 * Menu height on large screens, break point
	 */
	(function () {
		$( 'head' ).append( '<style type="text/css" id="main-nav-css"></style>' );

		var $menu = $( '.header__navigation' ),
			$css = $( '#main-nav-css' );

		if ( ! $menu || ! $css ) {
			return;
		}

		var isMultilineMenu = function () {
			return $menu.height() > 90;
		};

		var updateMenuStyle = function () {
			if ( Modernizr.mq( '(min-width: 992px)' ) && isMultilineMenu() ) {
				$css.text( '@media (min-width: 992px) { .header__container::before, .header__container::after { height: ' + $menu.height() + 'px; } }' );
			}
			else {
				$css.text( '' );
			}
		};

		updateMenuStyle();
		$( window ).on( 'resize', _.debounce( updateMenuStyle, 250 ) );
		$( window ).on( 'load', updateMenuStyle );
	})();


	/**
	 * Open position change text for the read more/close link.
	 */
	(function () {
		$( document ).on( 'shown.bs.collapse', '.js-open-position-collapse', function() {
			$( this ).parent().find( '.js-open-position-read-more-link' ).html( '<i class="fa fa-minus"></i>' + BoltsVars.close );
		});

		$( document ).on( 'hidden.bs.collapse', '.js-open-position-collapse', function() {
			$( this ).parent().find( '.js-open-position-read-more-link' ).html( '<i class="fa fa-plus"></i>' + BoltsVars.read_more );
		});
	}());

} );
