<?php
/**
 * The Header for Bolts Theme
 *
 * @package bolts-pt
 */

$theme_header = get_theme_mod( 'theme_header', 'kraft' );

?>

<!DOCTYPE html>
<html <?php language_attributes(); ?> class="no-js">
	<head>
		<meta charset="<?php bloginfo( 'charset' ); ?>" />
		<meta name="viewport" content="width=device-width, initial-scale=1.0" />
		<?php if ( is_singular() && pings_open( get_queried_object() ) ) : ?>
		<link rel="pingback" href="<?php bloginfo( 'pingback_url' ); ?>" />
		<?php endif; ?>

		<?php wp_head(); ?>
	</head>

	<body <?php body_class(); ?>>
	<div class="boxed-container">

	<header class="site-header  site-header--<?php echo esc_attr( $theme_header ); ?>"<?php echo empty( $bolts_style_attr ) ? '' : ' style="' . esc_attr( $bolts_style_attr ) . ';"'; ?>>

	<?php
	// Hide top bar, if both top sidebars are not in use.
	if ( is_active_sidebar( 'top-left-widgets' ) || is_active_sidebar( 'top-right-widgets' ) ) {
		get_template_part( 'template-parts/top-bar' );
	}
	?>

		<div class="header__container">
			<div class="container">
				<div class="header">
					<!-- Logo and site name -->
					<?php
						$bolts_logo       = get_theme_mod( 'logo_img', false );
						$bolts_logo2x     = get_theme_mod( 'logo2x_img', false );
						$bolts_logo_class = ! empty( $bolts_logo ) ? 'header__logo--image' : 'header__logo--text';
					?>
					<div class="header__logo  <?php echo esc_attr( $bolts_logo_class ); ?>">
						<a href="<?php echo esc_url( home_url( '/' ) ); ?>">
							<?php if ( ! empty( $bolts_logo ) ) : ?>
								<img src="<?php echo esc_url( $bolts_logo ); ?>" alt="<?php echo esc_attr( get_bloginfo( 'name' ) ); ?>" srcset="<?php echo esc_attr( $bolts_logo ); ?><?php echo empty( $bolts_logo2x ) ? '' : ', ' . esc_url( $bolts_logo2x ) . ' 2x'; ?>" class="img-fluid" <?php echo BoltsHelpers::get_logo_dimensions(); ?> />
							<?php
							else :
							?>
								<h1><?php bloginfo( 'name' ); ?></h1>
							<?php
							endif;
							?>
						</a>
					</div>
					<!-- Toggle button for Main Navigation on mobile -->
					<button class="btn  btn-dark  header__navbar-toggler  hidden-lg-up  js-sticky-mobile-option" type="button" data-toggle="collapse" data-target="#bolts-main-navigation"><i class="fa  fa-bars  hamburger"></i> <span><?php esc_html_e( 'MENU' , 'bolts-pt' ); ?></span></button>
					<?php if ( 'kraft' === $theme_header && is_active_sidebar( 'navigation-widgets' ) ) : ?>
						<!-- Header navigation widget area -->
						<div class="header__navigation-widgets">
							<?php dynamic_sidebar( apply_filters( 'bolts_navigation_widgets', 'navigation-widgets', get_the_ID() ) ); ?>
						</div>
					<?php endif; ?>
					<!-- Main Navigation -->
					<nav class="header__navigation  collapse  navbar-toggleable-md  js-sticky-desktop-option" id="bolts-main-navigation" aria-label="<?php esc_html_e( 'Main Menu', 'bolts-pt' ); ?>">
						<?php if ( 'kraft' === $theme_header && 'yes' === get_theme_mod( 'main_navigation_home_icon', 'yes' ) ) : ?>
							<!-- Home Icon in Navigation -->
							<a class="home-icon" href="<?php echo esc_url( home_url( '/' ) ); ?>">
								<i class="fa  fa-home"></i>
							</a>
						<?php endif; ?>
						<?php
						if ( has_nav_menu( 'main-menu' ) ) {
							wp_nav_menu( array(
								'theme_location' => 'main-menu',
								'container'      => false,
								'menu_class'     => 'main-navigation  js-main-nav  js-dropdown',
								'walker'         => new Aria_Walker_Nav_Menu(),
								'items_wrap'     => '<ul id="%1$s" class="%2$s" role="menubar">%3$s</ul>',
							) );
						}
						?>
					</nav>
					<?php if ( is_active_sidebar( 'header-widgets' ) ) : ?>
					<!-- Header widget area -->
					<div class="header__widgets">
						<?php dynamic_sidebar( apply_filters( 'bolts_header_widgets', 'header-widgets', get_the_ID() ) ); ?>
					</div>
					<?php endif; ?>
				</div>
			</div>
		</div>

	</header>
