<?php
/**
 * Customizer
 *
 * @package bolts-pt
 */

use ProteusThemes\CustomizerUtils\Setting;
use ProteusThemes\CustomizerUtils\Control;

/**
 * Contains methods for customizing the theme customization screen.
 *
 * @link http://codex.wordpress.org/Theme_Customization_API
 */
class Bolts_Customizer_Base {
	/**
	 * The singleton manager instance
	 *
	 * @see wp-includes/class-wp-customize-manager.php
	 * @var WP_Customize_Manager
	 */
	protected $wp_customize;

	/**
	 * Holds the array for the DynamicCSS.
	 */
	private $dynamic_css = array();

	public function __construct( WP_Customize_Manager $wp_manager ) {
		// Init the dynamic_css property.
		$this->dynamic_css = $this->dynamic_css_init();

		// Set the private propery to instance of wp_manager.
		$this->wp_customize = $wp_manager;

		// Register the settings/panels/sections/controls, main method.
		$this->register();

		/**
		 * Action and filters
		 */

		// Render the CSS and cache it to the theme_mod when the setting is saved.
		add_action( 'customize_save_after' , array( $this, 'cache_rendered_css' ) );

		// Save logo width/height dimensions.
		add_action( 'customize_save_logo_img' , array( __CLASS__, 'save_logo_dimensions' ), 10, 1 );

		// Handle the postMessage transfer method with some dynamically generated JS in the footer of the theme.
		add_action( 'customize_preview_init', array( $this, 'enqueue_customizer_js' ), 31 );
	}

	private function dynamic_css_init() {
		$darken5               = new Setting\DynamicCSS\ModDarken( 5 );
		$darken6               = new Setting\DynamicCSS\ModDarken( 6 );
		$darken13              = new Setting\DynamicCSS\ModDarken( 13 );
		$lighten3              = new Setting\DynamicCSS\ModLighten( 3 );
		$lighten6              = new Setting\DynamicCSS\ModLighten( 6 );
		$lighten9              = new Setting\DynamicCSS\ModLighten( 9 );
		$lighten13             = new Setting\DynamicCSS\ModLighten( 13 );
		$navigation_gradient   = new Setting\DynamicCSS\ModLinearGradient( $darken6, $orientation = 'to bottom' );
		$button_gradient       = new Setting\DynamicCSS\ModLinearGradient( $darken6, $orientation = 'to bottom' );
		$button_gradient_hover = new Setting\DynamicCSS\ModLinearGradient( $lighten6, $orientation = 'to top' );
		$is_font_output        = function ( $value ) {
			return 'off' !== $value;
		};

		return array(
			'top_bar_bg' => array(
				'default'    => '#e8e8e8',
				'css_props' => array( // List of all css properties this setting controls.
					array( // Each property in it's own array.
						'name'      => 'background-color',
						'selectors' => array(
							'noop' => array( // Regular selectors.
								'.top',
								'.top .widget_nav_menu .menu .sub-menu a',
							),
						),
					),
					array(
						'name'      => 'border-color',
						'selectors' => array(
							'noop' => array(
								'.top',
							),
						),
						'modifier'  => $lighten9,
					),
				),
			),

			'top_bar_color' => array(
				'default'    => '#999999',
				'css_props' => array(
					array(
						'name'      => 'color',
						'selectors' => array(
							'noop' => array(
								'.top',
								'.top-navigation a',
								'.top .widget_nav_menu .menu a',
								'.top .social-icons__link',
								'.top .icon-box__title',
								'.top .icon-box',
							),
						),
					),
					array(
						'name'      => 'color',
						'selectors' => array(
							'noop' => array(
								'.top-navigation a:focus',
								'.top-navigation a:hover',
								'.top .widget_nav_menu .menu a:focus',
								'.top .widget_nav_menu .menu a:hover',
								'.top .social-icons__link:focus',
								'.top .social-icons__link:hover',
							),
						),
						'modifier'  => $darken6,
					),
				),
			),

			'header_bg' => array(
				'default'    => '#f7f7f7',
				'css_props' => array(
					array(
						'name'      => 'background-color',
						'selectors' => array(
							'noop' => array(
								'.header__container',
							),
						),
					),
				),
			),

			'header_color' => array(
				'default'    => '#999999',
				'css_props' => array(
					array(
						'name'      => 'color',
						'selectors' => array(
							'noop' => array(
								'.header__widgets',
								'.header .icon-box__title',
							),
						),
					),
					array(
						'name'      => 'color',
						'selectors' => array(
							'noop' => array(
								'.header .icon-box .fa',
								'.header .icon-box__subtitle',
							),
						),
						'modifier'  => $darken13,
					),
					array(
						'name'      => 'color',
						'selectors' => array(
							'noop' => array(
								'.site-header--werk .header .icon-box__subtitle',
							),
						),
						'modifier'  => $lighten13,
					),
				),
			),

			'main_navigation_mobile_background' => array(
				'default' => '#ffffff',
				'css_props' => array(
					array(
						'name' => 'background-color',
						'selectors' => array(
							'@media (max-width: 991px)' => array(
								'.main-navigation',
								'.home-icon',
							),
						),
					),
					array(
						'name' => 'border-color',
						'selectors' => array(
							'@media (max-width: 991px)' => array(
								'.main-navigation',
								'.main-navigation a',
								'.home-icon',
							),
						),
						'modifier'  => $darken13,
					),
				),
			),

			'main_navigation_mobile_color' => array(
				'default' => '#333333',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'@media (max-width: 991px)' => array(
								'.main-navigation a',
								'.home-icon',
							),
						),
					),
				),
			),

			'main_navigation_mobile_color_hover' => array(
				'default' => '#000000',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'@media (max-width: 991px)' => array(
								'.main-navigation .menu-item:focus > a',
								'.main-navigation .menu-item:hover > a',
								'.home-icon:focus',
								'.home-icon:hover',
							),
						),
					),
				),
			),

			'main_navigation_mobile_sub_color_background' => array(
				'default' => '#eeeeee',
				'css_props' => array(
					array(
						'name' => 'background-color',
						'selectors' => array(
							'@media (max-width: 991px)' => array(
								'.main-navigation .sub-menu',
							),
						),
					),
				),
			),

			'main_navigation_mobile_sub_color' => array(
				'default' => '#999999',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'@media (max-width: 991px)' => array(
								'.main-navigation .sub-menu a',
							),
						),
					),
				),
			),

			'main_navigation_mobile_sub_color_hover' => array(
				'default' => '#333333',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'@media (max-width: 991px)' => array(
								'.main-navigation .sub-menu .menu-item:hover > a',
								'.main-navigation .sub-menu .menu-item:focus > a',
							),
						),
					),
				),
			),

			'main_navigation_bg' => array(
				'default' => '#292929',
				'css_props' => array(
					array(
						'name' => 'background-color',
						'selectors' => array(
							'@media (min-width: 992px)' => array(
								'.header__navigation',
								'.header__container::before',
								'.header__container::after',
								'.header__navigation-widgets',
							),
						),
					),
					array(
						'name'      => 'background',
						'selectors' => array(
							'@media (min-width: 992px)' => array(
								'.site-header--werk .header__navigation',
							),
						),
						'modifier'  => $navigation_gradient,
					),
				),
			),

			'main_navigation_color' => array(
				'default' => '#999999',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'@media (min-width: 992px)' => array(
								'.main-navigation a',
								'.main-navigation > .menu-item-has-children > a::after',
							),
						),
					),
					array(
						'name' => 'color',
						'selectors' => array(
							'@media (min-width: 992px)' => array(
								'.home-icon',
							),
						),
						'modifier'  => $lighten3,
					),
				),
			),

			'main_navigation_color_hover' => array(
				'default' => '#ffffff',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'@media (min-width: 992px)' => array(
								'.main-navigation > .menu-item:focus > a',
								'.main-navigation > .menu-item:hover > a',
								'.main-navigation > .current-menu-item > a',
								'.home-icon:focus',
								'.home-icon:hover',
								'.main-navigation .menu-item:focus > a::after',
								'.main-navigation .menu-item:hover > a::after',
								'.main-navigation > .current-menu-ancestor > a',
								'.main-navigation > .current-menu-ancestor.menu-item-has-children > a::after',
								'.main-navigation > .current-menu-item.menu-item-has-children > a::after',
							),
						),
					),
				),
			),

			'main_navigation_active_bg' => array(
				'default' => '#171717',
				'css_props' => array(
					array(
						'name' => 'background',
						'selectors' => array(
							'@media (min-width: 992px)' => array(
								'.site-header--werk .main-navigation > .current-menu-item > a',
								'.site-header--werk .main-navigation > .current-menu-ancestor > a',
							),
						),
						'modifier'  => $navigation_gradient,
					),
				),
			),

			'main_navigation_sub_bg' => array(
				'default' => '#3c3c3c',
				'css_props' => array(
					array(
						'name' => 'background-color',
						'selectors' => array(
							'@media (min-width: 992px)' => array(
								'.main-navigation .sub-menu a',
							),
						),
					),
					array(
						'name' => 'border-color',
						'selectors' => array(
							'@media (min-width: 992px)' => array(
								'.main-navigation .sub-menu a',
								'.main-navigation .sub-menu .sub-menu a',
							),
						),
						'modifier'  => $darken5,
					),
					array(
						'name' => 'color',
						'selectors' => array(
							'@media (min-width: 992px)' => array(
								'.main-navigation .sub-menu .menu-item-has-children::after',
							),
						),
						'modifier'  => $darken5,
					),
				),
			),

			'main_navigation_sub_bg_hover' => array(
				'default' => '#2f2f2f',
				'css_props' => array(
					array(
						'name' => 'background-color',
						'selectors' => array(
							'@media (min-width: 992px)' => array(
								'.main-navigation .sub-menu .menu-item > a:hover',
							),
						),
					),
				),
			),

			'main_navigation_sub_color' => array(
				'default' => '#999999',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'@media (min-width: 992px)' => array(
								'.main-navigation .sub-menu .menu-item a',
							),
						),
					),
				),
			),

			'main_navigation_sub_color_hover' => array(
				'default' => '#ffffff',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'@media (min-width: 992px)' => array(
								'.main-navigation .sub-menu .menu-item > a:focus',
								'.main-navigation .sub-menu .menu-item > a:hover',
							),
						),
					),
				),
			),

			'main_navigation_separator_color' => array(
				'default' => '#666666',
				'css_props' => array(
					array(
						'name' => 'background-color',
						'selectors' => array(
							'@media (min-width: 992px)' => array(
								'.site-header--werk .main-navigation > .menu-item::after',
							),
						),
					),
				),
			),

			'page_header_bg_color' => array(
				'default' => '#f7f7f7',
				'css_props' => array(
					array(
						'name' => 'background-color',
						'selectors' => array(
							'noop' => array(
								'.page-header',
							),
						),
					),
					array(
						'name' => 'border-color',
						'selectors' => array(
							'noop' => array(
								'.page-header',
							),
						),
						'modifier'  => $darken5,
					),
				),
			),

			'page_header_color' => array(
				'default' => '#333333',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'noop' => array(
								'.page-header__title',
								'.page-header__subtitle::before',
							),
						),
					),
				),
			),

			'page_header_subtitle_color' => array(
				'default' => '#999999',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'noop' => array(
								'.page-header__subtitle',
							),
						),
					),
				),
			),

			'breadcrumbs_color' => array(
				'default'    => '#999999',
				'css_props' => array(
					array(
						'name'      => 'color',
						'selectors' => array(
							'noop' => array(
								'.breadcrumbs a',
								'.breadcrumbs a::after',
							),
						),
					),
				),
			),

			'breadcrumbs_color_hover' => array(
				'default'    => '#333333',
				'css_props' => array(
					array(
						'name'      => 'color',
						'selectors' => array(
							'noop' => array(
								'.breadcrumbs a:focus',
								'.breadcrumbs a:hover',
							),
						),
					),
				),
			),

			'breadcrumbs_color_active' => array(
				'default'    => '#333333',
				'css_props' => array(
					array(
						'name'      => 'color',
						'selectors' => array(
							'noop' => array(
								'.breadcrumbs .current-item',
							),
						),
					),
				),
			),

			'text_color_content_area' => array(
				'default' => '#999999',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'noop' => array(
								'.content-area',
								'.hentry__content .icon-box',
							),
						),
					),
				),
			),

			'headings_color' => array(
				'default' => '#333333',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'noop' => array(
								'h1',
								'h2',
								'h3',
								'h4',
								'h5',
								'h6',
								'hentry__title',
								'.hentry__title a',
								'.latest-news--block .latest-news__title a',
								'.latest-news--more-news',
								'.person-profile__tag',
								'.step__title',
								'.testimonial__author-name',
								'.page-box__title a',
								'.sidebar__headings',
								'body.woocommerce-page ul.products li.product h3',
								'.woocommerce ul.products li.product h3',
								'body.woocommerce-page .entry-summary .entry-title',
							),
						),
					),
				),
			),

			'primary_color' => array(
				'default' => '#fec303',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'noop' => array(
								'body.woocommerce-page ul.products li.product .price',
								'.woocommerce ul.products li.product .price',
								'body.woocommerce-page ul.products li.product a:hover img',
								'.woocommerce ul.products li.product a:hover img',
								'body.woocommerce-page ul.products li.product a',
								'.woocommerce ul.products li.product a',
								'body.woocommerce-page div.product p.price',
								'.site-header--werk .header .icon-box .fa',
							),
						),
					),
					array(
						'name' => 'background-color',
						'selectors' => array(
							'noop' => array(
								'.btn-primary',
								'.portfolio-grid__card-label',
								'.pricing-list__badge',
								'.latest-news--block .latest-news__date',
								'.widget_calendar caption',
								'.pagination .current',
								'.pagination .page-numbers:focus',
								'.pagination .page-numbers:hover',
								'.portfolio__label',
								'.main-navigation a::before',
								'.home-icon::after',
								'.featured-widget::before',
								'.step__number::before',
								'.portfolio-grid__arrow::before',
								'.title-with-button__button::before',
								'.testimonial__carousel::before',
								'.portfolio__navigation-item::before',
								'body.woocommerce-page div.product form.cart .button.single_add_to_cart_button',
								'body.woocommerce-page #review_form #respond input#submit',
								'body.woocommerce-page .woocommerce-error a.button',
								'body.woocommerce-page .woocommerce-info a.button',
								'body.woocommerce-page .woocommerce-message a.button',
								'.woocommerce button.button.alt:disabled',
								'.woocommerce button.button.alt:disabled:hover',
								'.woocommerce button.button.alt:disabled[disabled]',
								'.woocommerce button.button.alt:disabled[disabled]:hover',
								'.woocommerce-cart .wc-proceed-to-checkout a.checkout-button',
								'body.woocommerce-page #payment #place_order',
								'body.woocommerce-page nav.woocommerce-pagination ul li span.current',
							),
						),
					),
					array(
						'name' => 'background-color',
						'selectors' => array(
							'@media (min-width: 992px)' => array(
								'.portfolio-grid__nav-item::before',
							),
						),
					),
					array(
						'name' => 'background-color',
						'selectors' => array(
							'noop' => array(
								'.btn-primary:focus',
								'.btn-primary:hover',
								'body.woocommerce-page div.product form.cart .button.single_add_to_cart_button:focus',
								'body.woocommerce-page div.product form.cart .button.single_add_to_cart_button:hover',
								'body.woocommerce-page #review_form #respond input#submit:hover',
								'.woocommerce-cart .wc-proceed-to-checkout a.checkout-button:hover',
								'body.woocommerce-page #payment #place_order:hover',
								'body.woocommerce-page .woocommerce-error a.button:hover',
								'body.woocommerce-page .woocommerce-info a.button:hover',
								'body.woocommerce-page .woocommerce-message a.button:hover',
							),
						),
						'modifier'  => $darken6,
					),
					array(
						'name' => 'background-color',
						'selectors' => array(
							'noop' => array(
								'.btn-primary:active:hover',
								'.btn-primary:active:focus',
								'.btn-primary:active.focus',
								'.btn-primary.active.focus',
								'.btn-primary.active:focus',
								'.btn-primary.active:hover',
							),
						),
						'modifier'  => $darken13,
					),
					array(
						'name' => 'background',
						'selectors' => array(
							'noop' => array(
								'body.woocommerce-page .widget_price_filter .ui-slider .ui-slider-handle',
								'body.woocommerce-page .widget_price_filter .ui-slider .ui-slider-range',
							),
						),
					),
					array(
						'name' => 'border-color',
						'selectors' => array(
							'noop' => array(
								'.btn-primary',
								'.testimonial__author',
								'.footer-featured',
							),
						),
					),
					array(
						'name' => 'border-top-color',
						'selectors' => array(
							'noop' => array(
								'.widget_search .search-field:focus',
								'.person-profile__container',
								'.form-control:focus',
								'body.woocommerce-page .quantity .qty:focus',
								'body.woocommerce-page div.product .woocommerce-tabs ul.tabs li.active a',
							),
						),
					),
					array(
						'name' => 'border-left-color',
						'selectors' => array(
							'noop' => array(
								'.sidebar .widget_nav_menu .menu > li.current-menu-item > a',
								'.sidebar .widget_nav_menu .menu > li > a:focus',
								'.sidebar .widget_nav_menu .menu > li > a:hover',
								'.hentry__content .widget_nav_menu .menu > li.current-menu-item > a',
								'.hentry__content .widget_nav_menu .menu > li > a:focus',
								'.hentry__content .widget_nav_menu .menu > li > a:hover',
								'body.woocommerce-page .widget_product_categories .product-categories > li.current-cat > a',
								'body.woocommerce-page .widget_product_categories .product-categories > li > a:focus',
								'body.woocommerce-page .widget_product_categories .product-categories > li > a:hover',
							),
						),
					),
					array(
						'name' => 'border-color',
						'selectors' => array(
							'noop' => array(
								'.btn-primary:focus',
								'.btn-primary:hover',
							),
						),
						'modifier'  => $darken6,
					),
					array(
						'name' => 'border-color',
						'selectors' => array(
							'noop' => array(
								'.btn-primary:active:hover',
								'.btn-primary:active:focus',
								'.btn-primary:active.focus',
								'.btn-primary.active.focus',
								'.btn-primary.active:focus',
								'.btn-primary.active:hover',
							),
						),
						'modifier'  => $darken13,
					),
				),
			),

			'secondary_color' => array(
				'default' => '#3baed4',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'noop' => array(
								'.accordion__panel .panel-title a:hover',
								'.accordion .more-link:focus',
								'.accordion .more-link:hover',
								'a.person-profile__social-icon:focus',
								'a.person-profile__social-icon:hover',
								'.open-position__content .read-more',
								'.open-position__content .read-more:focus',
								'body.woocommerce-page .star-rating, .woocommerce .star-rating',
								'.hentry__content a.icon-box:focus .fa',
								'.hentry__content a.icon-box:hover .fa',
							),
						),
					),
					array(
						'name' => 'background-color',
						'selectors' => array(
							'noop' => array(
								'.btn-secondary',
								'.brochure-box',
								'.brochure-box:focus',
							),
						),
					),
					array(
						'name' => 'background-color',
						'selectors' => array(
							'noop' => array(
								'.btn-secondary:focus',
								'.btn-secondary:hover',
								'.brochure-box:hover',
							),
						),
						'modifier'  => $darken6,
					),
					array(
						'name' => 'background-color',
						'selectors' => array(
							'noop' => array(
								'.btn-secondary:active:hover',
								'.btn-secondary:active:focus',
								'.btn-secondary:active.focus',
								'.btn-secondary.active.focus',
								'.btn-secondary.active:focus',
								'.btn-secondary.active:hover',
							),
						),
						'modifier'  => $darken13,
					),
					array(
						'name' => 'border-color',
						'selectors' => array(
							'noop' => array(
								'.btn-secondary',
							),
						),
					),
					array(
						'name' => 'border-color',
						'selectors' => array(
							'noop' => array(
								'.btn-secondary:focus',
								'.btn-secondary:hover',
							),
						),
						'modifier'  => $darken6,
					),
					array(
						'name' => 'border-color',
						'selectors' => array(
							'noop' => array(
								'.btn-secondary:active:hover',
								'.btn-secondary:active:focus',
								'.btn-secondary:active.focus',
								'.btn-secondary.active.focus',
								'.btn-secondary.active:focus',
								'.btn-secondary.active:hover',
							),
						),
						'modifier'  => $darken13,
					),
				),
			),

			'link_color' => array(
				'default' => '#3baed4',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'noop' => array(
								'a',
							),
						),
					),
					array(
						'name' => 'color',
						'selectors' => array(
							'noop' => array(
								'a:focus',
								'a:hover',
							),
						),
						'modifier'  => $darken6,
					),
				),
			),

			'dark_button_color' => array(
				'default' => '#4a4a4a',
				'css_props' => array(
					array(
						'name' => 'background',
						'selectors' => array(
							'noop' => array(
								'.btn-dark',
							),
						),
						'modifier'  => $button_gradient,
					),
					array(
						'name' => 'background',
						'selectors' => array(
							'@media (min-width: 992px)' => array(
								'.home-icon',
							),
						),
						'modifier'  => $button_gradient,
					),
					array(
						'name' => 'background',
						'selectors' => array(
							'noop' => array(
								'.btn-dark:focus',
								'.btn-dark:hover',
							),
						),
						'modifier'  => $button_gradient_hover,
					),
					array(
						'name' => 'background',
						'selectors' => array(
							'@media (min-width: 992px)' => array(
								'.home-icon:focus',
								'.home-icon:hover',
							),
						),
						'modifier'  => $button_gradient_hover,
					),
					array(
						'name' => 'border-top-color',
						'selectors' => array(
							'noop' => array(
								'.btn-dark',
								'.btn-dark:focus',
								'.btn-dark:hover',
							),
						),
						'modifier'  => $lighten6,
					),
					array(
						'name' => 'border-top-color',
						'selectors' => array(
							'@media (min-width: 992px)' => array(
								'.home-icon',
								'.home-icon:focus',
								'.home-icon:hover',
							),
						),
						'modifier'  => $lighten6,
					),
				),
			),

			'light_button_color' => array(
				'default' => '#ffffff',
				'css_props' => array(
					array(
						'name' => 'background-color',
						'selectors' => array(
							'noop' => array(
								'.btn-light',
							),
						),
					),
					array(
						'name' => 'background-color',
						'selectors' => array(
							'noop' => array(
								'.btn-light:focus',
								'.btn-light:hover',
							),
						),
						'modifier'  => $darken6,
					),
				),
			),

			'footer_bg_color' => array(
				'default' => '#f7f7f7',
				'css_props' => array(
					array(
						'name' => 'background-color',
						'selectors' => array(
							'noop' => array(
								'.footer-top',
							),
						),
					),
				),
			),

			'footer_title_color' => array(
				'default' => '#333333',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'noop' => array(
								'.footer-top__heading',
							),
						),
					),
				),
			),

			'footer_text_color' => array(
				'default' => '#999999',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'noop' => array(
								'.footer-top',
							),
						),
					),
				),
			),

			'footer_link_color' => array(
				'default' => '#999999',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'noop' => array(
								'.footer-top a',
								'.footer-top .widget_nav_menu .menu a',
							),
						),
					),
				),
			),

			'footer_bottom_bg_color' => array(
				'default' => '#e8e8e8',
				'css_props' => array(
					array(
						'name' => 'background-color',
						'selectors' => array(
							'noop' => array(
								'.footer-bottom__container',
							),
						),
					),
					array(
						'name' => 'border-color',
						'selectors' => array(
							'noop' => array(
								'.footer-bottom__container',
							),
						),
						'modifier'  => $darken5,
					),
				),
			),

			'footer_bottom_text_color' => array(
				'default' => '#999999',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'noop' => array(
								'.footer-bottom',
							),
						),
					),
				),
			),

			'footer_bottom_link_color' => array(
				'default' => '#333333',
				'css_props' => array(
					array(
						'name' => 'color',
						'selectors' => array(
							'noop' => array(
								'.footer-bottom a',
							),
						),
					),
				),
			),

			'primary_font' => array(
				'default' => 'Roboto',
				'transport' => 'refresh',
				'active_callback' => $is_font_output,
				'css_props' => array(
					array(
						'name' => 'font-family',
						'selectors' => array(
							'noop' => array(
								'body',
								'.main-navigation .sub-menu a',
								'body.woocommerce-page div.product form.cart .single_variation .price del',
							),
						),
					),
				),
			),

			'secondary_font' => array(
				'default' => 'Roboto',
				'transport' => 'refresh',
				'active_callback' => $is_font_output,
				'css_props' => array(
					array(
						'name' => 'font-family',
						'selectors' => array(
							'noop' => array(
								'h1',
								'h2',
								'h3',
								'h4',
								'h5',
								'h6',
								'.h1',
								'.h2',
								'.h3',
								'.h4',
								'.h5',
								'.h6',
								'.breadcrumbs',
								'.footer-top__headings',
								'.portfolio__price',
								'.portfolio__label',
								'.portfolio__specification',
								'.main-navigation a',
								'.jumbotron-content__title',
								'.header .icon-box__subtitle',
								'.skype-button__title',
								'.open-position__content .read-more',
								'.brochure-box__text',
								'.testimonial__author-name',
								'.testimonial__carousel',
								'.step__title',
								'.step__number',
								'.person-profile__tag',
								'.accordion__panel .panel-title a',
								'.number-counter__number',
								'.pricing-list__price',
								'.portfolio-grid__nav-item',
								'.portfolio-grid__card-label',
								'.portfolio-grid__card-price',
								'.hero-columns__cta',
								'.title-with-button__button',
								'.widget_categories a',
								'.widget_meta a',
							),
						),
					),
				),
			),

		);
	}

	/**
	 * This hooks into 'customize_register' (available as of WP 3.4) and allows
	 * you to add new sections and controls to the Theme Customize screen.
	 *
	 * Note: To enable instant preview, we have to actually write a bit of custom
	 * javascript. See live_preview() for more.
	 *
	 * @see add_action('customize_register',$func)
	 */
	public function register() {
		/**
		 * Settings
		 */

		// Branding.
		$this->wp_customize->add_setting( 'logo_img', array( 'sanitize_callback' => 'esc_url' ) );
		$this->wp_customize->add_setting( 'logo2x_img', array( 'sanitize_callback' => 'esc_url' ) );
		$this->wp_customize->add_setting( 'logo_top_margin', array( 'default' => 27, 'sanitize_callback' => 'absint' ) );

		// Header.
		$this->wp_customize->add_setting( 'top_bar_visibility', array( 'default' => 'yes', 'sanitize_callback' => 'sanitize_key' ) );
		$this->wp_customize->add_setting( 'theme_header', array( 'default' => 'kraft', 'sanitize_callback' => 'sanitize_key' ) );
		$this->wp_customize->add_setting( 'header_bg_img', array( 'sanitize_callback' => 'esc_url' ) );
		$this->wp_customize->add_setting( 'header_bg_img_repeat', array( 'default' => 'repeat', 'sanitize_callback' => 'sanitize_key' ) );
		$this->wp_customize->add_setting( 'header_bg_img_position_x', array( 'default' => 'left', 'sanitize_callback' => 'sanitize_key' ) );
		$this->wp_customize->add_setting( 'header_bg_img_attachment', array( 'default' => 'scroll', 'sanitize_callback' => 'sanitize_key' ) );

		// Navigation.
		$this->wp_customize->add_setting( 'main_navigation_home_icon', array( 'default' => 'yes', 'sanitize_callback' => 'sanitize_key' ) );

		// Page header area.
		$this->wp_customize->add_setting( 'page_header_bg_img', array( 'sanitize_callback' => 'esc_url' ) );
		$this->wp_customize->add_setting( 'page_header_bg_img_repeat', array( 'default' => 'repeat', 'sanitize_callback' => 'sanitize_key' ) );
		$this->wp_customize->add_setting( 'page_header_bg_img_position_x', array( 'default' => 'left', 'sanitize_callback' => 'sanitize_key' ) );
		$this->wp_customize->add_setting( 'page_header_bg_img_attachment', array( 'default' => 'scroll', 'sanitize_callback' => 'sanitize_key' ) );
		$this->wp_customize->add_setting( 'show_page_title_area', array( 'default' => 'yes', 'sanitize_callback' => 'sanitize_key' ) );

		// Typography.
		$this->wp_customize->add_setting( 'charset_setting', array( 'default' => 'latin', 'sanitize_callback' => 'sanitize_key' ) );

		// Theme layout & color.
		$this->wp_customize->add_setting( 'layout_mode', array( 'default' => 'wide', 'sanitize_callback' => 'sanitize_key' ) );

		// Shop.
		if ( BoltsHelpers::is_woocommerce_active() ) {
			$this->wp_customize->add_setting( 'products_per_page', array( 'default' => 9, 'sanitize_callback' => 'absint' ) );
			$this->wp_customize->add_setting( 'single_product_sidebar', array( 'default' => 'left', 'sanitize_callback' => 'sanitize_key' ) );
		}

		// Portfolio.
		if ( BoltsHelpers::is_portfolio_plugin_active() ) {
			$this->wp_customize->add_setting( 'portfolio_parent_page', array( 'default' => 0, 'sanitize_callback' => 'sanitize_key' ) );
			$this->wp_customize->add_setting( 'portfolio_gallery_columns', array( 'default' => 2, 'sanitize_callback' => 'absint' ) );
			$this->wp_customize->add_setting( 'portfolio_slug', array(
				'default'           => 'portfolio',
				'sanitize_callback' => array( $this, 'sanitize_portfolio_slug' ),
			) );
		}

		// Footer.
		$this->wp_customize->add_setting( 'footer_widgets_layout', array( 'default' => '[4,6,8]', 'sanitize_callback' => 'wp_kses_post' ) );

		$this->wp_customize->add_setting( 'footer_bottom_left_txt', array( 'sanitize_callback' => 'wp_kses_post', 'default' => '<a href="https://www.proteusthemes.com/wordpress-themes/bolts/">Bolts Theme</a> - Made by ProteusThemes.' ) );
		$this->wp_customize->add_setting( 'footer_bottom_right_txt', array( 'sanitize_callback' => 'wp_kses_post', 'default' => 'Copyright &copy; ' . date( 'Y' ) . ' Bolts. All rights reserved.' ) );

		// Custom code (css/js).
		$this->wp_customize->add_setting( 'custom_js_head', array( 'sanitize_callback' => array( 'ProteusThemes\CustomizerUtils\Helpers', 'wp_kses_script' ) ) );
		$this->wp_customize->add_setting( 'custom_js_footer', array( 'sanitize_callback' => array( 'ProteusThemes\CustomizerUtils\Helpers', 'wp_kses_script' ) ) );
		$this->wp_customize->add_setting( 'custom_css', array( 'default' => '', 'sanitize_callback' => 'strip_tags' ) );

		// Migrate any existing theme CSS to the core option added in WordPress 4.7.
		if ( function_exists( 'wp_update_custom_css_post' ) ) {
			$css = get_theme_mod( 'custom_css', '' );

			if ( ! empty( $css ) ) {
				$core_css = wp_get_custom_css(); // Preserve any CSS already added to the core option.
				$return   = wp_update_custom_css_post( '/* Migrated CSS from old Theme Custom CSS setting: */' . PHP_EOL . $css . PHP_EOL . PHP_EOL . '/* New custom CSS: */' . PHP_EOL . $core_css );
				if ( ! is_wp_error( $return ) ) {
					// Remove the old theme_mod, so that the CSS is stored in only one place moving forward.
					remove_theme_mod( 'custom_css' );
				}
			}

			// Add new "CSS setting" that will only notify the users that the new "Additional CSS" field is available.
			// It can't be the same name ('custom_css'), because the core control is also named 'custom_css' and it would not display the WP core "Additional CSS" control.
			$this->wp_customize->add_setting( 'pt_custom_css', array( 'default' => '', 'sanitize_callback' => 'strip_tags' ) );
		}

		// ACF.
		$this->wp_customize->add_setting( 'show_acf', array( 'default' => 'no', 'sanitize_callback' => 'sanitize_key' ) );
		$this->wp_customize->add_setting( 'use_minified_css', array( 'default' => 'no', 'sanitize_callback' => 'sanitize_key' ) );

		// All the DynamicCSS settings.
		foreach ( $this->dynamic_css as $setting_id => $args ) {
			$this->wp_customize->add_setting(
				new Setting\DynamicCSS( $this->wp_customize, $setting_id, $args )
			);
		}

		/**
		 * Panel and Sections
		 */

		// One ProteusThemes panel to rule them all.
		$this->wp_customize->add_panel( 'panel_bolts', array(
			'title'       => esc_html__( '[PT] Theme Options', 'bolts-pt' ),
			'description' => esc_html__( 'All Bolts theme specific settings.', 'bolts-pt' ),
			'priority'    => 10,
		) );

		// Individual sections.
		$this->wp_customize->add_section( 'bolts_section_logos', array(
			'title'       => esc_html__( 'Logo', 'bolts-pt' ),
			'description' => esc_html__( 'Logo for the Bolts theme.', 'bolts-pt' ),
			'priority'    => 10,
			'panel'       => 'panel_bolts',
		) );

		// Header.
		$this->wp_customize->add_section( 'bolts_section_header', array(
			'title'       => esc_html__( 'Header', 'bolts-pt' ),
			'description' => esc_html__( 'All layout and appearance settings for the header.', 'bolts-pt' ),
			'priority'    => 20,
			'panel'       => 'panel_bolts',
		) );

		$this->wp_customize->add_section( 'bolts_section_navigation', array(
			'title'       => esc_html__( 'Navigation', 'bolts-pt' ),
			'description' => esc_html__( 'Navigation for the Bolts theme.', 'bolts-pt' ),
			'priority'    => 30,
			'panel'       => 'panel_bolts',
		) );

		$this->wp_customize->add_section( 'bolts_section_page_header', array(
			'title'       => esc_html__( 'Page Header Area', 'bolts-pt' ),
			'description' => esc_html__( 'All layout and appearance settings for the page header area (regular pages).', 'bolts-pt' ),
			'priority'    => 33,
			'panel'       => 'panel_bolts',
		) );

		$this->wp_customize->add_section( 'bolts_section_breadcrumbs', array(
			'title'       => esc_html__( 'Breadcrumbs', 'bolts-pt' ),
			'description' => esc_html__( 'All layout and appearance settings for breadcrumbs.', 'bolts-pt' ),
			'priority'    => 35,
			'panel'       => 'panel_bolts',
		) );

		$this->wp_customize->add_section( 'bolts_section_theme_colors', array(
			'title'       => esc_html__( 'Theme Layout &amp; Colors', 'bolts-pt' ),
			'priority'    => 40,
			'panel'       => 'panel_bolts',
		) );

		$this->wp_customize->add_section( 'bolts_section_fonts', array(
			'title'       => esc_html__( 'Fonts', 'bolts-pt' ),
			'priority'    => 50,
			'panel'       => 'panel_bolts',
		) );

		if ( BoltsHelpers::is_woocommerce_active() ) {
			$this->wp_customize->add_section( 'bolts_section_shop', array(
				'title'       => esc_html__( 'Shop', 'bolts-pt' ),
				'priority'    => 80,
				'panel'       => 'panel_bolts',
			) );
		}

		if ( BoltsHelpers::is_portfolio_plugin_active() ) {
			$this->wp_customize->add_section( 'bolts_section_portfolio', array(
				'title'       => esc_html__( 'Portfolio', 'bolts-pt' ),
				'priority'    => 85,
				'panel'       => 'panel_bolts',
			) );
		}

		$this->wp_customize->add_section( 'bolts_section_footer', array(
			'title'       => esc_html__( 'Footer', 'bolts-pt' ),
			'description' => esc_html__( 'All layout and appearance settings for the footer.', 'bolts-pt' ),
			'priority'    => 90,
			'panel'       => 'panel_bolts',
		) );

		$this->wp_customize->add_section( 'bolts_section_custom_code', array(
			'title'       => esc_html__( 'Custom Code' , 'bolts-pt' ),
			'priority'    => 100,
			'panel'       => 'panel_bolts',
		) );

		$this->wp_customize->add_section( 'bolts_section_other', array(
			'title'       => esc_html__( 'Other' , 'bolts-pt' ),
			'priority'    => 150,
			'panel'       => 'panel_bolts',
		) );

		/**
		 * Controls
		 */

		// Section: bolts_section_logos.
		$this->wp_customize->add_control( new WP_Customize_Image_Control(
			$this->wp_customize,
			'logo_img',
			array(
				'label'       => esc_html__( 'Logo Image', 'bolts-pt' ),
				'description' => esc_html__( 'Max height for the logo image is 120px.', 'bolts-pt' ),
				'section'     => 'bolts_section_logos',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Image_Control(
			$this->wp_customize,
			'logo2x_img',
			array(
				'label'       => esc_html__( 'Retina Logo Image', 'bolts-pt' ),
				'description' => esc_html__( '2x logo size, for screens with high DPI.', 'bolts-pt' ),
				'section'     => 'bolts_section_logos',
			)
		) );
		$this->wp_customize->add_control(
			'logo_top_margin',
			array(
				'type'        => 'number',
				'label'       => esc_html__( 'Logo top margin', 'bolts-pt' ),
				'description' => esc_html__( 'In pixels.', 'bolts-pt' ),
				'section'     => 'bolts_section_logos',
				'input_attrs' => array(
					'min'  => 0,
					'max'  => 120,
					'step' => 5,
				),
			)
		);

		// Section: header.
		$this->wp_customize->add_control( 'theme_header', array(
			'type'        => 'select',
			'priority'    => 5,
			'label'       => esc_html__( 'Header', 'bolts-pt' ),
			'description' => esc_html__( 'Select which header you want to use in the theme.', 'bolts-pt' ),
			'section'     => 'bolts_section_header',
			'choices'     => array(
				'kraft' => esc_html__( 'Kraft (default)', 'bolts-pt' ),
				'werk'  => esc_html__( 'Werk', 'bolts-pt' ),
			),
		) );
		$this->wp_customize->add_control( new WP_Customize_Image_Control(
			$this->wp_customize,
			'header_bg_img',
			array(
				'priority' => 10,
				'label'    => esc_html__( 'Header Background Image', 'bolts-pt' ),
				'section'  => 'bolts_section_header',
			)
		) );
		$this->wp_customize->add_control( 'header_bg_img_repeat', array(
			'priority'        => 11,
			'label'           => esc_html__( 'Header Background Repeat', 'bolts-pt' ),
			'section'         => 'bolts_section_header',
			'type'            => 'radio',
			'active_callback' => array( $this, 'is_header_bg_img_set' ),
			'choices'         => array(
				'no-repeat' => esc_html__( 'No Repeat', 'bolts-pt' ),
				'repeat'    => esc_html__( 'Tile', 'bolts-pt' ),
				'repeat-x'  => esc_html__( 'Tile Horizontally', 'bolts-pt' ),
				'repeat-y'  => esc_html__( 'Tile Vertically', 'bolts-pt' ),
			),
		) );
		$this->wp_customize->add_control( 'header_bg_img_position_x', array(
			'priority'        => 12,
			'label'           => esc_html__( 'Header Background Position', 'bolts-pt' ),
			'section'         => 'bolts_section_header',
			'type'            => 'radio',
			'active_callback' => array( $this, 'is_header_bg_img_set' ),
			'choices'         => array(
				'left'   => esc_html__( 'Left', 'bolts-pt' ),
				'center' => esc_html__( 'Center', 'bolts-pt' ),
				'right'  => esc_html__( 'Right', 'bolts-pt' ),
			),
		) );
		$this->wp_customize->add_control( 'header_bg_img_attachment', array(
			'priority'        => 13,
			'label'           => esc_html__( 'Header Background Attachment', 'bolts-pt' ),
			'section'         => 'bolts_section_header',
			'type'            => 'radio',
			'active_callback' => array( $this, 'is_header_bg_img_set' ),
			'choices'         => array(
				'scroll' => esc_html__( 'Scroll', 'bolts-pt' ),
				'fixed'  => esc_html__( 'Fixed', 'bolts-pt' ),
			),
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'header_bg',
			array(
				'priority' => 14,
				'label'    => esc_html__( 'Header background color', 'bolts-pt' ),
				'section'  => 'bolts_section_header',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'header_color',
			array(
				'priority' => 15,
				'label'    => esc_html__( 'Header color', 'bolts-pt' ),
				'section'  => 'bolts_section_header',
			)
		) );
		$this->wp_customize->add_control( 'top_bar_visibility', array(
			'type'        => 'select',
			'priority'    => 20,
			'label'       => esc_html__( 'Top bar visibility', 'bolts-pt' ),
			'description' => esc_html__( 'Show or hide?', 'bolts-pt' ),
			'section'     => 'bolts_section_header',
			'choices'     => array(
				'yes'         => esc_html__( 'Show', 'bolts-pt' ),
				'no'          => esc_html__( 'Hide', 'bolts-pt' ),
				'hide_mobile' => esc_html__( 'Hide on Mobile', 'bolts-pt' ),
			),
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'top_bar_bg',
			array(
				'priority' => 21,
				'label'    => esc_html__( 'Top bar background color', 'bolts-pt' ),
				'section'  => 'bolts_section_header',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'top_bar_color',
			array(
				'priority' => 22,
				'label'    => esc_html__( 'Top bar text color', 'bolts-pt' ),
				'section'  => 'bolts_section_header',
			)
		) );

		// Section: bolts_section_navigation.
		$this->wp_customize->add_control( 'main_navigation_home_icon', array(
			'type'            => 'select',
			'priority'        => 110,
			'label'           => esc_html__( 'Home Icon', 'bolts-pt' ),
			'section'         => 'bolts_section_navigation',
			'active_callback' => array( $this, 'is_kraft_header_set' ),
			'choices'         => array(
				'yes' => esc_html__( 'Show', 'bolts-pt' ),
				'no'  => esc_html__( 'Hide', 'bolts-pt' ),
			),
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'main_navigation_bg',
			array(
				'priority' => 125,
				'label'    => esc_html__( 'Main navigation background color', 'bolts-pt' ),
				'section'  => 'bolts_section_navigation',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'main_navigation_color',
			array(
				'priority' => 130,
				'label'    => esc_html__( 'Main navigation link color', 'bolts-pt' ),
				'section'  => 'bolts_section_navigation',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'main_navigation_color_hover',
			array(
				'priority' => 132,
				'label'    => esc_html__( 'Main navigation link hover color', 'bolts-pt' ),
				'section'  => 'bolts_section_navigation',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'main_navigation_active_bg',
			array(
				'priority'        => 135,
				'label'           => esc_html__( 'Main navigation active background color', 'bolts-pt' ),
				'section'         => 'bolts_section_navigation',
				'active_callback' => array( $this, 'is_werk_header_set' ),
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'main_navigation_sub_bg',
			array(
				'priority' => 160,
				'label'    => esc_html__( 'Main navigation submenu background', 'bolts-pt' ),
				'section'  => 'bolts_section_navigation',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'main_navigation_sub_bg_hover',
			array(
				'priority' => 165,
				'label'    => esc_html__( 'Main navigation submenu background hover color', 'bolts-pt' ),
				'section'  => 'bolts_section_navigation',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'main_navigation_sub_color',
			array(
				'priority' => 170,
				'label'    => esc_html__( 'Main navigation submenu link color', 'bolts-pt' ),
				'section'  => 'bolts_section_navigation',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'main_navigation_sub_color_hover',
			array(
				'priority' => 175,
				'label'    => esc_html__( 'Main navigation submenu link hover color', 'bolts-pt' ),
				'section'  => 'bolts_section_navigation',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'main_navigation_mobile_color',
			array(
				'priority' => 190,
				'label'    => esc_html__( 'Main navigation link color (mobile)', 'bolts-pt' ),
				'section'  => 'bolts_section_navigation',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'main_navigation_mobile_color_hover',
			array(
				'priority' => 192,
				'label'    => esc_html__( 'Main navigation link hover color (mobile)', 'bolts-pt' ),
				'section'  => 'bolts_section_navigation',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'main_navigation_mobile_sub_color_background',
			array(
				'priority' => 193,
				'label'    => esc_html__( 'Main navigation submenu background color (mobile)', 'bolts-pt' ),
				'section'  => 'bolts_section_navigation',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'main_navigation_mobile_sub_color',
			array(
				'priority' => 194,
				'label'    => esc_html__( 'Main navigation submenu link color (mobile)', 'bolts-pt' ),
				'section'  => 'bolts_section_navigation',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'main_navigation_mobile_sub_color_hover',
			array(
				'priority' => 195,
				'label'    => esc_html__( 'Main navigation submenu link hover color (mobile)', 'bolts-pt' ),
				'section'  => 'bolts_section_navigation',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'main_navigation_mobile_background',
			array(
				'priority' => 188,
				'label'    => esc_html__( 'Main navigation background color (mobile)', 'bolts-pt' ),
				'section'  => 'bolts_section_navigation',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'main_navigation_separator_color',
			array(
				'priority'        => 200,
				'label'           => esc_html__( 'Main navigation separator color', 'bolts-pt' ),
				'section'         => 'bolts_section_navigation',
				'active_callback' => array( $this, 'is_werk_header_set' ),
			)
		) );

		// Section: bolts_section_page_header.
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'page_header_bg_color',
			array(
				'priority' => 10,
				'label'    => esc_html__( 'Page Header background color', 'bolts-pt' ),
				'section'  => 'bolts_section_page_header',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Image_Control(
			$this->wp_customize,
			'page_header_bg_img',
			array(
				'priority' => 20,
				'label'    => esc_html__( 'Page Header Background Image', 'bolts-pt' ),
				'section'  => 'bolts_section_page_header',
			)
		) );
		$this->wp_customize->add_control( 'page_header_bg_img_repeat', array(
			'priority'        => 21,
			'label'           => esc_html__( 'Page Header Background Repeat', 'bolts-pt' ),
			'section'         => 'bolts_section_page_header',
			'type'            => 'radio',
			'active_callback' => array( $this, 'is_page_header_bg_img_set' ),
			'choices'         => array(
				'no-repeat'  => esc_html__( 'No Repeat', 'bolts-pt' ),
				'repeat'     => esc_html__( 'Tile', 'bolts-pt' ),
				'repeat-x'   => esc_html__( 'Tile Horizontally', 'bolts-pt' ),
				'repeat-y'   => esc_html__( 'Tile Vertically', 'bolts-pt' ),
			),
		) );
		$this->wp_customize->add_control( 'page_header_bg_img_position_x', array(
			'priority'        => 22,
			'label'           => esc_html__( 'Page Header Background Position', 'bolts-pt' ),
			'section'         => 'bolts_section_page_header',
			'type'            => 'radio',
			'active_callback' => array( $this, 'is_page_header_bg_img_set' ),
			'choices'         => array(
				'left'       => esc_html__( 'Left', 'bolts-pt' ),
				'center'     => esc_html__( 'Center', 'bolts-pt' ),
				'right'      => esc_html__( 'Right', 'bolts-pt' ),
			),
		) );
		$this->wp_customize->add_control( 'page_header_bg_img_attachment', array(
			'priority'        => 23,
			'label'           => esc_html__( 'Page Header Background Attachment', 'bolts-pt' ),
			'section'         => 'bolts_section_page_header',
			'type'            => 'radio',
			'active_callback' => array( $this, 'is_page_header_bg_img_set' ),
			'choices'         => array(
				'scroll'     => esc_html__( 'Scroll', 'bolts-pt' ),
				'fixed'      => esc_html__( 'Fixed', 'bolts-pt' ),
			),
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'page_header_color',
			array(
				'priority' => 30,
				'label'    => esc_html__( 'Page Header title color', 'bolts-pt' ),
				'section'  => 'bolts_section_page_header',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'page_header_subtitle_color',
			array(
				'priority' => 31,
				'label'    => esc_html__( 'Page Header subtitle color', 'bolts-pt' ),
				'section'  => 'bolts_section_page_header',
			)
		) );
		$this->wp_customize->add_control( 'show_page_title_area', array(
			'type'        => 'select',
			'priority'    => 35,
			'label'       => esc_html__( 'Show page title area', 'bolts-pt' ),
			'description' => esc_html__( 'This will hide the page title area on all pages. You can also hide individual page headers in page settings. To remove breadcrumbs from all pages, please deactivate the Breadcrumb NavXT plugin.', 'bolts-pt' ),
			'section'     => 'bolts_section_page_header',
			'choices'     => array(
				'yes' => esc_html__( 'Show', 'bolts-pt' ),
				'no'  => esc_html__( 'Hide', 'bolts-pt' ),
			),
		) );

		// Section: bolts_section_breadcrumbs.
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'breadcrumbs_color',
			array(
				'priority' => 45,
				'label'    => esc_html__( 'Breadcrumbs text color', 'bolts-pt' ),
				'section'  => 'bolts_section_breadcrumbs',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'breadcrumbs_color_hover',
			array(
				'priority' => 50,
				'label'    => esc_html__( 'Breadcrumbs hover text color', 'bolts-pt' ),
				'section'  => 'bolts_section_breadcrumbs',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'breadcrumbs_color_active',
			array(
				'priority' => 50,
				'label'    => esc_html__( 'Breadcrumbs active text color', 'bolts-pt' ),
				'section'  => 'bolts_section_breadcrumbs',
			)
		) );

		// Section: bolts_section_theme_colors.
		$this->wp_customize->add_control( 'layout_mode', array(
			'type'     => 'select',
			'priority' => 10,
			'label'    => esc_html__( 'Layout', 'bolts-pt' ),
			'section'  => 'bolts_section_theme_colors',
			'choices'  => array(
				'wide'  => esc_html__( 'Wide', 'bolts-pt' ),
				'boxed' => esc_html__( 'Boxed', 'bolts-pt' ),
			),
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'text_color_content_area',
			array(
				'priority' => 30,
				'label'    => esc_html__( 'Text color', 'bolts-pt' ),
				'section'  => 'bolts_section_theme_colors',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'link_color',
			array(
				'priority' => 35,
				'label'    => esc_html__( 'Link color', 'bolts-pt' ),
				'section'  => 'bolts_section_theme_colors',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'headings_color',
			array(
				'priority' => 33,
				'label'    => esc_html__( 'Headings color', 'bolts-pt' ),
				'section'  => 'bolts_section_theme_colors',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'primary_color',
			array(
				'priority' => 34,
				'label'    => esc_html__( 'Primary color', 'bolts-pt' ),
				'section'  => 'bolts_section_theme_colors',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'secondary_color',
			array(
				'priority' => 35,
				'label'    => esc_html__( 'Secondary color', 'bolts-pt' ),
				'section'  => 'bolts_section_theme_colors',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'dark_button_color',
			array(
				'priority' => 36,
				'label'    => esc_html__( 'Dark button background color', 'bolts-pt' ),
				'section'  => 'bolts_section_theme_colors',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'light_button_color',
			array(
				'priority' => 37,
				'label'    => esc_html__( 'Light button background color', 'bolts-pt' ),
				'section'  => 'bolts_section_theme_colors',
			)
		) );

		// Section: bolts_section_fonts.
		$this->wp_customize->add_control( 'primary_font', array(
			'type'        => 'select',
			'label'       => esc_html__( 'Primary Font', 'bolts-pt' ),
			'section'     => 'bolts_section_fonts',
			'description' => esc_html__( 'Used in main content.', 'bolts-pt' ),
			'choices'     => array(
				'Roboto'    => 'Roboto',
				'Open Sans' => 'Open Sans',
				'off'       => esc_html__( 'None (do not use Google Fonts)', 'bolts-pt' ),
			),
		) );
		$this->wp_customize->add_control( 'secondary_font', array(
			'type'        => 'select',
			'label'       => esc_html__( 'Secondary Font', 'bolts-pt' ),
			'section'     => 'bolts_section_fonts',
			'description' => esc_html__( 'Used for headings.', 'bolts-pt' ),
			'choices'     => array(
				'Roboto'     => 'Roboto',
				'Montserrat' => 'Montserrat',
				'off'        => esc_html__( 'None (do not use Google Fonts)', 'bolts-pt' ),
			),
		) );
		$this->wp_customize->add_control( 'charset_setting', array(
			'type'     => 'select',
			'label'    => esc_html__( 'Character set for Google Fonts', 'bolts-pt' ),
			'section'  => 'bolts_section_fonts',
			'choices'  => array(
				'latin'        => 'Latin',
				'latin-ext'    => 'Latin Extended',
				'cyrillic'     => 'Cyrillic',
				'cyrillic-ext' => 'Cyrillic Extended',
				'vietnamese'   => 'Vietnamese',
				'greek'        => 'Greek',
				'greek-ext'    => 'Greek Extended',
			),
		) );

		// Section: bolts_section_shop.
		if ( BoltsHelpers::is_woocommerce_active() ) {
			$this->wp_customize->add_control( 'products_per_page', array(
					'label'   => esc_html__( 'Number of products per page', 'bolts-pt' ),
					'section' => 'bolts_section_shop',
				)
			);
			$this->wp_customize->add_control( 'single_product_sidebar', array(
					'label'   => esc_html__( 'Sidebar on single product page', 'bolts-pt' ),
					'section' => 'bolts_section_shop',
					'type'    => 'select',
					'choices' => array(
						'none'  => esc_html__( 'No sidebar', 'bolts-pt' ),
						'left'  => esc_html__( 'Left', 'bolts-pt' ),
						'right' => esc_html__( 'Right', 'bolts-pt' ),
					),
				)
			);
		}

		// Section: bolts_section_portfolio.
		if ( BoltsHelpers::is_portfolio_plugin_active() ) {
			$this->wp_customize->add_control( 'portfolio_parent_page', array(
					'label'       => esc_html__( 'Portfolio parent page', 'bolts-pt' ),
					'description' => esc_html__( 'Page with all the portfolio items.', 'bolts-pt' ),
					'section'     => 'bolts_section_portfolio',
					'type'        => 'dropdown-pages',
				)
			);
			$this->wp_customize->add_control( 'portfolio_slug', array(
					'label'       => esc_html__( 'Portfolio slug', 'bolts-pt' ),
					'description' => esc_html__( 'This is used in the URL part. After changing this, you must save the permalink settings again in Settings &rarr; Permalinks.', 'bolts-pt' ),
					'section'     => 'bolts_section_portfolio',
				)
			);
			$this->wp_customize->add_control( 'portfolio_gallery_columns', array(
					'label'   => esc_html__( 'Number of columns in gallery', 'bolts-pt' ),
					'section' => 'bolts_section_portfolio',
					'type'    => 'select',
					'choices' => array(
						'1' => '1',
						'2' => '2',
						'3' => '3',
					),
				)
			);
		}

		// Section: bolts_section_footer.
		$this->wp_customize->add_control( new Control\LayoutBuilder(
			$this->wp_customize,
			'footer_widgets_layout',
			array(
				'priority'    => 1,
				'label'       => esc_html__( 'Footer widgets layout', 'bolts-pt' ),
				'description' => esc_html__( 'Select number of widget you want in the footer and then with the slider rearrange the layout', 'bolts-pt' ),
				'section'     => 'bolts_section_footer',
				'input_attrs' => array(
					'min'     => 0,
					'max'     => 12,
					'step'    => 1,
					'maxCols' => 6,
				),
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'footer_bg_color',
			array(
				'priority' => 10,
				'label'    => esc_html__( 'Footer background color', 'bolts-pt' ),
				'section'  => 'bolts_section_footer',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'footer_title_color',
			array(
				'priority' => 30,
				'label'    => esc_html__( 'Footer widget title color', 'bolts-pt' ),
				'section'  => 'bolts_section_footer',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'footer_text_color',
			array(
				'priority' => 31,
				'label'    => esc_html__( 'Footer text color', 'bolts-pt' ),
				'section'  => 'bolts_section_footer',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'footer_link_color',
			array(
				'priority' => 32,
				'label'    => esc_html__( 'Footer link color', 'bolts-pt' ),
				'section'  => 'bolts_section_footer',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'footer_bottom_bg_color',
			array(
				'priority' => 35,
				'label'    => esc_html__( 'Footer bottom background color', 'bolts-pt' ),
				'section'  => 'bolts_section_footer',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'footer_bottom_text_color',
			array(
				'priority' => 36,
				'label'    => esc_html__( 'Footer bottom text color', 'bolts-pt' ),
				'section'  => 'bolts_section_footer',
			)
		) );
		$this->wp_customize->add_control( new WP_Customize_Color_Control(
			$this->wp_customize,
			'footer_bottom_link_color',
			array(
				'priority' => 37,
				'label'    => esc_html__( 'Footer bottom link color', 'bolts-pt' ),
				'section'  => 'bolts_section_footer',
			)
		) );
		$this->wp_customize->add_control( 'footer_bottom_left_txt', array(
			'type'        => 'text',
			'priority'    => 110,
			'label'       => esc_html__( 'Footer bottom text on the left', 'bolts-pt' ),
			'description' => esc_html__( 'You can use HTML: a, span, i, em, strong, img.', 'bolts-pt' ),
			'section'     => 'bolts_section_footer',
		) );
		$this->wp_customize->add_control( 'footer_bottom_right_txt', array(
			'type'        => 'text',
			'priority'    => 115,
			'label'       => esc_html__( 'Footer bottom text on the right', 'bolts-pt' ),
			'description' => esc_html__( 'You can use HTML: a, span, i, em, strong, img.', 'bolts-pt' ),
			'section'     => 'bolts_section_footer',
		) );

		// Section: bolts_section_custom_code.
		if ( function_exists( 'wp_update_custom_css_post' ) ) {
			// Show the notice of custom CSS setting migration.
			$this->wp_customize->add_control( 'pt_custom_css', array(
				'type'        => 'hidden',
				'label'       => esc_html__( 'Custom CSS', 'bolts-pt' ),
				'description' => esc_html__( 'This field is obsolete. The existing code was migrated to the "Additional CSS" field, that can be found in the root of the customizer. This new "Additional CSS" field is a WordPress core field and was introduced in WP version 4.7.', 'bolts-pt' ),
				'section'     => 'bolts_section_custom_code',
			) );
		}
		else {
			$this->wp_customize->add_control( 'custom_css', array(
				'type'        => 'textarea',
				'label'       => esc_html__( 'Custom CSS', 'bolts-pt' ),
				'description' => sprintf( esc_html__( '%s How to find CSS classes %s in the theme.', 'bolts-pt' ), '<a href="https://www.youtube.com/watch?v=V2aAEzlvyDc" target="_blank">', '</a>' ),
				'section'     => 'bolts_section_custom_code',
			) );
		}
		$this->wp_customize->add_control( 'custom_js_head', array(
			'type'        => 'textarea',
			'label'       => esc_html__( 'Custom JavaScript (head)', 'bolts-pt' ),
			'description' => esc_html__( 'You have to include the &lt;script&gt;&lt;/script&gt; tags as well. Paste your Google Analytics tracking code here.', 'bolts-pt' ),
			'section'     => 'bolts_section_custom_code',
		) );
		$this->wp_customize->add_control( 'custom_js_footer', array(
			'type'        => 'textarea',
			'label'       => esc_html__( 'Custom JavaScript (footer)', 'bolts-pt' ),
			'description' => esc_html__( 'You have to include the &lt;script&gt;&lt;/script&gt; tags as well.', 'bolts-pt' ),
			'section'     => 'bolts_section_custom_code',
		) );

		// Section: bolts_section_other.
		$this->wp_customize->add_control( 'show_acf', array(
			'type'        => 'select',
			'label'       => esc_html__( 'Show ACF admin panel?', 'bolts-pt' ),
			'description' => esc_html__( 'If you want to use ACF and need the ACF admin panel set this to <strong>Yes</strong>. Do not change if you do not know what you are doing.', 'bolts-pt' ),
			'section'     => 'bolts_section_other',
			'choices'     => array(
				'no'  => esc_html__( 'No', 'bolts-pt' ),
				'yes' => esc_html__( 'Yes', 'bolts-pt' ),
			),
		) );
		$this->wp_customize->add_control( 'use_minified_css', array(
			'type'    => 'select',
			'label'   => esc_html__( 'Use minified theme CSS', 'bolts-pt' ),
			'section' => 'bolts_section_other',
			'choices' => array(
				'no'  => esc_html__( 'No', 'bolts-pt' ),
				'yes' => esc_html__( 'Yes', 'bolts-pt' ),
			),
		) );
	}

	/**
	 * Returns true if Kraft header is selected.
	 *
	 * Used by the main_navigation_home_icon control.
	 *
	 * @return boolean
	 */
	public function is_kraft_header_set() {
		return 'kraft' === get_theme_mod( 'theme_header', 'kraft' );
	}

	/**
	 * Returns true if Werk header is selected.
	 *
	 * Used by the main_navigation_separator_color and main_navigation_active_bg control.
	 *
	 * @return boolean
	 */
	public function is_werk_header_set() {
		return 'werk' === get_theme_mod( 'theme_header', 'kraft' );
	}

	/**
	 * Returns true if header background image is set.
	 *
	 * Used by the header_bg_img_* controls.
	 *
	 * @return boolean
	 */
	public function is_header_bg_img_set() {
		$header_bg_img = get_theme_mod( 'header_bg_img', '' );
		return ! empty( $header_bg_img );
	}

	/**
	 * Returns true if page header background image is set.
	 *
	 * Used by the page_header_bg_img_* controls.
	 *
	 * @return boolean
	 */
	public function is_page_header_bg_img_set() {
		$page_header_bg_img = get_theme_mod( 'page_header_bg_img', '' );
		return ! empty( $page_header_bg_img );
	}

	/**
	 * Cache the rendered CSS after the settings are saved in the DB.
	 * This is purely a performance improvement.
	 *
	 * Used by hook: add_action( 'customize_save_after' , array( $this, 'cache_rendered_css' ) );
	 *
	 * @return void
	 */
	public function cache_rendered_css() {
		set_theme_mod( 'cached_css', $this->render_css() );
	}

	/**
	 * Get the dimensions of the logo image when the setting is saved.
	 * This is purely a performance improvement.
	 *
	 * Used by hook: add_action( 'customize_save_logo_img' , array( $this, 'save_logo_dimensions' ), 10, 1 );
	 *
	 * @return void
	 */
	public static function save_logo_dimensions( $setting ) {
		$logo_width_height = array();
		$img_data          = getimagesize( esc_url( $setting->post_value() ) );

		if ( is_array( $img_data ) ) {
			$logo_width_height = array_slice( $img_data, 0, 2 );
			$logo_width_height = array_combine( array( 'width', 'height' ), $logo_width_height );
		}

		set_theme_mod( 'logo_dimensions_array', $logo_width_height );
	}

	/**
	 * Render the CSS from all the settings which are of type `Setting\DynamicCSS`
	 *
	 * @return string text/css
	 */
	public function render_css() {
		$out = array();

		foreach ( $this->get_dynamic_css_settings() as $setting ) {
			$out[] = $setting->render_css();
		}

		return implode( PHP_EOL, $out );
	}

	/**
	 * Get only the CSS settings of type `Setting\DynamicCSS`.
	 *
	 * @see is_dynamic_css_setting
	 * @return array
	 */
	public function get_dynamic_css_settings() {
		return array_filter( $this->wp_customize->settings(), array( $this, 'is_dynamic_css_setting' ) );
	}

	/**
	 * Helper conditional function for filtering the settings.
	 *
	 * @param  mixed $setting
	 * @return boolean
	 */
	protected function is_dynamic_css_setting( $setting ) {
		return is_a( $setting, '\ProteusThemes\CustomizerUtils\Setting\DynamicCSS' );
	}

	/**
	 * Enqueue the JS for live preview the settings of type `Setting\DynamicCSS`.
	 *
	 * All the color changes are transported to the live preview frame using the 'postMessage'
	 * method. The settings key IDs, selectors and CSS props are passed to the script.
	 *
	 * Used by hook: 'customize_preview_init'
	 *
	 * @see add_action('customize_preview_init',$func)
	 */
	public function enqueue_customizer_js() {
		wp_enqueue_script(
			'bolts-live-customize',
			get_template_directory_uri() . '/vendor/proteusthemes/wp-customizer-utilities/assets/live-customize.js',
			array( 'jquery', 'customize-preview' ),
			false,
			true
		);

		wp_localize_script( 'bolts-live-customize', 'ptCustomizerDynamicCSS', $this->js_dynamic_css() );
	}

	/**
	 * Prepare suitable data to pass to JS - only selectors, setting IDs and name of
	 * CSS properties. All selectors are returned, not taking MQ in account.
	 *
	 * @return array
	 */
	private function js_dynamic_css() {
		$out = array();

		foreach ( $this->dynamic_css as $setting_id => $setting ) {
			foreach ( $setting['css_props'] as $css_prop ) {
				$css_selectors = array();

				foreach ( $css_prop['selectors'] as $selectors ) {
					$css_selectors += $selectors;
				}

				$css_selectors = array_filter( $css_selectors );
				$css_selectors = array_unique( $css_selectors );

				if ( ! empty( $css_selectors ) ) {
					$out[] = array(
						'settingID' => $setting_id,
						'selectors' => join( ', ', $css_selectors ),
						'cssProp'   => $css_prop['name'],
					);
				}
			}
		}

		return $out;
	}

	/**
	 * Create a sanitized slug, but always with a fallback to portfolio if it is empty
	 *
	 * @param  string $title
	 * @return string
	 */
	public static function sanitize_portfolio_slug( $title ) {
		return sanitize_title( $title, 'portfolio' );
	}
}
