<?php
/**
 * Filters for Bolts WP theme
 *
 * @package bolts-pt
 */

/**
 * BoltsFilters class with filter hooks
 */
class BoltsFilters {

	/**
	 * Runs on class initialization. Adds filters and actions.
	 */
	function __construct() {

		// ProteusWidgets.
		add_filter( 'pw/widget_views_path', array( $this, 'set_widgets_view_path' ) );
		add_filter( 'pw/testimonial_widget', array( $this, 'set_testimonial_settings' ) );
		add_filter( 'pw/featured_page_widget_page_box_image_size', array( $this, 'set_page_box_image_size' ) );
		add_filter( 'pw/featured_page_widget_inline_image_size', array( $this, 'set_inline_image_size' ) );
		add_filter( 'pw/featured_page_excerpt_lengths', array( $this, 'set_featured_page_excerpt_lengths' ) );
		add_filter( 'pw/social_icons_fa_icons_list', array( $this, 'social_icons_fa_icons_list' ) );
		add_filter( 'pw/default_social_icon', array( $this, 'default_social_icon' ) );
		add_filter( 'pw/featured_page_fields', array( $this, 'set_featured_page_fields' ) );
		add_filter( 'pw/steps_widget_settings', array( $this, 'set_steps_widget_settings' ) );

		// Custom tag font size.
		add_filter( 'widget_tag_cloud_args', array( $this, 'set_tag_cloud_sizes' ) );

		// Custom text after excerpt.
		add_filter( 'excerpt_more', array( $this, 'excerpt_more' ) );

		// Footer widgets with dynamic layouts.
		add_filter( 'dynamic_sidebar_params', array( $this, 'footer_widgets_params' ), 9, 1 );

		// Google fonts.
		add_filter( 'bolts_pre_google_web_fonts', array( $this, 'additional_fonts' ) );
		add_filter( 'bolts_subsets_google_web_fonts', array( $this, 'subsets_google_web_fonts' ) );

		// Page builder.
		add_filter( 'siteorigin_panels_widget_style_fields', array( $this, 'add_fields_to_pagebuilder_widget_panel' ) );
		add_filter( 'siteorigin_panels_widget_style_attributes', array( $this, 'add_attributes_to_pagebuilder_widget_panel' ), 10, 2 );
		add_filter( 'siteorigin_panels_settings_defaults', array( $this, 'siteorigin_panels_settings_defaults' ) );
		add_filter( 'siteorigin_panels_widgets', array( $this, 'add_icons_to_page_builder_for_pw_widgets' ), 15 );
		add_filter( 'siteorigin_panels_widget_dialog_tabs', array( $this, 'siteorigin_panels_add_widgets_dialog_tabs' ), 15 );

		// SiteOrigin custom widgets.
		add_filter( 'siteorigin_widgets_widget_folders', array( $this, 'siteorigin_widgets_widget_folders' ) );
		add_filter( 'siteorigin_widgets_template_file_pw_open_position', array( $this, 'siteorigin_widgets_template_file_pw_open_position' ) );
		add_filter( 'admin_init', array( $this, 'siteorigin_widgets_active_widgets' ) );

		// Embeds.
		add_filter( 'embed_oembed_html', array( $this, 'embed_oembed_html' ), 10, 1 );

		// Protocols.
		add_filter( 'kses_allowed_protocols', array( $this, 'kses_allowed_protocols' ) );

		// One Click Demo Import plugin.
		add_filter( 'pt-ocdi/import_files', array( $this, 'ocdi_import_files' ) );
		add_action( 'pt-ocdi/after_import', array( $this, 'ocdi_after_import_setup' ) );
		add_filter( 'pt-ocdi/message_after_file_fetching_error', array( $this, 'ocdi_message_after_file_fetching_error' ) );

		// <body> classes
		add_filter( 'body_class', array( $this, 'body_classes' ) );

		// Shortcodes plugin.
		add_filter( 'pt/widget_text_do_shortcode', '__return_true' );

		// Sticky menu
		add_filter( 'pt-sticky-menu/theme_panel', array( $this, 'pt_sticky_menu_theme_panel' ) );
		add_filter( 'pt-sticky-menu/mobile_menu_button_class', array( $this, 'pt_sticky_menu_mobile_menu_button_class' ) );

		// Remove references to SiteOrigin Premium.
		add_filter( 'siteorigin_premium_upgrade_teaser', '__return_false' );
	}


	/**
	 * Filter the Testimonial widget fields that the Bolts theme will need from ProteusWidgets - Tesimonial widget.
	 *
	 * @param array $attr default attributes.
	 * @return array
	 */
	function set_testimonial_settings( $attr ) {
		$attr['number_of_testimonial_per_slide'] = 2;
		$attr['rating']                          = false;
		$attr['author_description']              = true;
		$attr['author_avatar']                   = false;
		$attr['bootstrap_version']               = 4;
		return $attr;
	}

	/**
	 * Custom tag font size.
	 *
	 * @param array $args default arguments.
	 * @return array
	 */
	function set_tag_cloud_sizes( $args ) {
		$args['smallest'] = 14;
		$args['largest']  = 14;
		$args['unit'] = 'px';
		return $args;
	}


	/**
	 * Custom text after excerpt
	 *
	 * @param array $more default more value.
	 * @return array
	 */
	function excerpt_more( $more ) {
		return ' &hellip;';
	}


	/**
	 * Filter the dynamic sidebars and alter the BS col classes for the footer wigets
	 *
	 * @param  array $params parameters of the sidebar.
	 * @return array
	 */
	function footer_widgets_params( $params ) {
		static $counter              = 0;
		static $first_row            = true;
		$footer_widgets_layout_array = BoltsHelpers::footer_widgets_layout_array();

		if ( 'footer-widgets' === $params[0]['id'] ) {
			// 'before_widget' contains __col-num__, see inc/theme-sidebars.php.
			$params[0]['before_widget'] = str_replace( '__col-num__', $footer_widgets_layout_array[ $counter ], $params[0]['before_widget'] );

			// First widget in the any non-first row.
			if ( false === $first_row && 0 === $counter ) {
				$params[0]['before_widget'] = '</div><div class="row">' . $params[0]['before_widget'];
			}

			$counter++;
		}

		end( $footer_widgets_layout_array );
		if ( $counter > key( $footer_widgets_layout_array ) ) {
			$counter   = 0;
			$first_row = false;
		}

		return $params;
	}


	/**
	 * Filter setting ProteusWidgets mustache widget views path for Bolts
	 */
	function set_widgets_view_path() {
		return get_template_directory() . '/inc/widgets-views';
	}


	/**
	 * Filter the Featured page widget pw-page-box image size for Bolts (ProteusWidgets)
	 *
	 * @param array $image default image parameters.
	 * @return array
	 */
	function set_page_box_image_size( $image ) {
		$image['width']  = 360;
		$image['height'] = 180;
		return $image;
	}


	/**
	 * Filter the Featured page widget pw-page-box image size for Bolts (ProteusWidgets)
	 *
	 * @param array $icon default FA icon string.
	 * @return string
	 */
	function default_social_icon( $icon ) {
		return 'fa-linkedin-square';
	}


	/**
	 * Filter the Featured page widget pw-inline image size for Bolts (ProteusWidgets)
	 *
	 * @param array $image default image parameters.
	 * @return array
	 */
	function set_inline_image_size( $image ) {
		$image['width']  = 100;
		$image['height'] = 70;
		return $image;
	}

	/**
	 * Filter the Featured page widget excerpt lengths for Bolts (ProteusWidgets)
	 *
	 * @param array $excerpt_lengths default excerpt lengths.
	 * @return array
	 */
	function set_featured_page_excerpt_lengths( $excerpt_lengths ) {
		$excerpt_lengths['inline_excerpt'] = 55;
		$excerpt_lengths['block_excerpt']  = 140;
		return $excerpt_lengths;
	}

	/**
	 * Filter for the list of Font-Awesome icons in social icons widget in Bolts (ProteusWidgets)
	 */
	function social_icons_fa_icons_list() {
		return array(
			'fa-facebook',
			'fa-twitter',
			'fa-youtube',
			'fa-google-plus',
			'fa-pinterest',
			'fa-tumblr',
			'fa-xing',
			'fa-vimeo',
			'fa-linkedin',
			'fa-rss',
			'fa-github',
			'fa-bitbucket',
		);
	}

	/**
	 * Return Google fonts and sizes
	 *
	 * @see https://github.com/grappler/wp-standard-handles/blob/master/functions.php
	 * @param array $fonts google fonts used in the theme.
	 * @return array Google fonts and sizes.
	 */
	public static function additional_fonts( $fonts = array() ) {
		$primary_fonts = $secondary_fonts = array();

		$primary_font   = get_theme_mod( 'primary_font', 'Roboto' );
		$secondary_font = get_theme_mod( 'secondary_font', 'Roboto' );

		switch ( $primary_font ) {
			case 'off': // Fonts disabled.
				break;

			case 'Roboto':
				$primary_fonts[ $primary_font ] = array( '300', '400', '700' );
				break;

			default:
				$primary_fonts[ $primary_font ] = array( '400', '700' );
				break;
		}

		switch ( $secondary_font ) {
			case 'off': // Fonts disabled.
				break;

			case 'Roboto':
				$secondary_fonts[ $secondary_font ] = array( '900' );
				break;

			default:
				$secondary_fonts[ $secondary_font ] = array( '700' );
				break;
		}

		$fonts = array_merge_recursive( $fonts, $primary_fonts, $secondary_fonts );

		// Remove duplicated weights.
		$fonts = array_map( 'array_unique', $fonts );

		return $fonts;
	}


	/**
	 * Add subsets from customizer, if needed.
	 *
	 * @param array $subsets google font subsets used in the theme.
	 * @return array
	 */
	function subsets_google_web_fonts( $subsets ) {
		$additional_subset = get_theme_mod( 'charset_setting', 'latin' );

		array_push( $subsets, $additional_subset );

		return $subsets;
	}


	/**
	 * Add the "featured widget" checkbox to the PageBuilder widget panel under Design section.
	 *
	 * @link https://siteorigin.com/docs/page-builder/hooks/custom-row-settings/
	 * @param array $fields Array of all existing (default) PageBuilder widget settings fields.
	 * @return array
	 */
	function add_fields_to_pagebuilder_widget_panel( $fields ) {
		$fields['featured_widgets'] = array(
			'name'     => esc_html__( 'Widget Style', 'bolts-pt' ),
			'type'     => 'checkbox',
			'label'    => esc_html__( 'Set a box around the widget', 'bolts-pt' ),
			'group'    => 'design',
			'priority' => 17,
		);

		return $fields;
	}


	/**
	 * Add the functionality of the above "featured widget" checkbox
	 *
	 * @link https://siteorigin.com/docs/page-builder/hooks/custom-row-settings/
	 * @param array $attributes Array of all attributes that get applied to the widget on front-end.
	 * @param array $args Array of all settings from the widget panel.
	 * @return array
	 */
	function add_attributes_to_pagebuilder_widget_panel( $attributes, $args ) {
		if ( empty( $attributes['class'] ) ) {
			$attributes['class'] = array();
		}

		if ( ! empty( $args['featured_widgets'] ) ) {
			$attributes['class'][] = 'featured-widget';
		}

		return $attributes;
	}


	/**
	 * Embedded videos and video container around them
	 *
	 * @param string $html html to be enclosed with responsive HTML.
	 * @return string
	 */
	function embed_oembed_html( $html ) {
		if (
			false !== strstr( $html, 'youtube.com' ) ||
			false !== strstr( $html, 'wordpress.tv' ) ||
			false !== strstr( $html, 'wordpress.com' ) ||
			false !== strstr( $html, 'vimeo.com' )
		) {
			$out = '<div class="embed-responsive  embed-responsive-16by9">' . $html . '</div>';
		} else {
			$out = $html;
		}
		return $out;
	}


	/**
	 * Add more allowed protocols
	 *
	 * @param array $protocols default protocols.
	 * @return array
	 * @link https://developer.wordpress.org/reference/functions/wp_allowed_protocols/
	 */
	static function kses_allowed_protocols( $protocols ) {
		return array_merge( $protocols, array( 'skype' ) );
	}


	/**
	 * Change the default settings for SO
	 *
	 * @param  array $settings default Page Builder settings.
	 * @return array
	 */
	function siteorigin_panels_settings_defaults( $settings ) {
		$settings['title-html']           = '<h3 class="widget-title"><span class="widget-title__inline">{{title}}</span></h3>';
		$settings['full-width-container'] = '.boxed-container';
		$settings['mobile-width']         = '991';
		$settings['post-types'][]         = 'portfolio';

		return $settings;
	}


	/**
	 * Set SiteOrigin custom widgets folder
	 *
	 * @param array $folders default SO widget folders.
	 * @return array
	 */
	function siteorigin_widgets_widget_folders( $folders ) {
		$folders[] = get_template_directory() . '/inc/so-widgets/';
		return $folders;
	}


	/**
	 * Set SiteOrigin template for Open Position widget
	 *
	 * @param string $template_file default SO widget template file.
	 * @return string
	 */
	function siteorigin_widgets_template_file_pw_open_position( $template_file ) {
		return get_template_directory() . '/inc/so-widgets/widget-open-position/tpl/open-position-template.php';
	}


	/**
	 * Activate SiteOrigin custom widgets
	 */
	function siteorigin_widgets_active_widgets() {
		if ( ! get_theme_mod( 'so_custom_widgets_activated' ) && class_exists( 'SiteOrigin_Widgets_Bundle' ) ) {
			SiteOrigin_Widgets_Bundle::single()->activate_widget( 'widget-open-position' );
			set_theme_mod( 'so_custom_widgets_activated', true );
		}
	}


	/**
	 * Set Featured Page widget fields (ProteusWidgets)
	 *
	 * @param array $fields default settings for Featured page widget.
	 * @return array
	 */
	function set_featured_page_fields( $fields ) {
		$fields['show_read_more_link'] = true;
		return $fields;
	}


	/**
	 * Set Steps widget fields (ProteusWidgets)
	 *
	 * @param array $fields default settings for Steps widget.
	 * @return array
	 */
	function set_steps_widget_settings( $fields ) {
		$fields['use_icons'] = false;
		return $fields;
	}


	/**
	 * Define demo import files for One Click Demo Import plugin.
	 */
	function ocdi_import_files() {
		return array(
			array(
				'import_file_name'           => 'Bolts Construction',
				'import_file_url'            => 'http://artifacts.proteusthemes.com/xml-exports/bolts-latest.xml',
				'import_widget_file_url'     => 'http://artifacts.proteusthemes.com/json-widgets/bolts-construction-ptcs.json',
				'import_customizer_file_url' => 'http://artifacts.proteusthemes.com/customizer-exports/bolts-construction.dat',
				'import_preview_image_url'   => 'http://artifacts.proteusthemes.com/import-preview-images/bolts-construction.jpg',
				'preview_url'                => 'https://demo.proteusthemes.com/bolts',
			),
			array(
				'import_file_name'           => 'Bolts Transport',
				'import_file_url'            => 'http://artifacts.proteusthemes.com/xml-exports/bolts-transport-latest.xml',
				'import_widget_file_url'     => 'http://artifacts.proteusthemes.com/json-widgets/bolts-transport-ptcs.json',
				'import_customizer_file_url' => 'http://artifacts.proteusthemes.com/customizer-exports/bolts-transport.dat',
				'import_preview_image_url'   => 'http://artifacts.proteusthemes.com/import-preview-images/bolts-transport.jpg',
				'preview_url'                => 'https://demo.proteusthemes.com/bolts-transport',
			),
			array(
				'import_file_name'           => 'Bolts Factory',
				'import_file_url'            => 'http://artifacts.proteusthemes.com/xml-exports/bolts-factory-latest.xml',
				'import_widget_file_url'     => 'http://artifacts.proteusthemes.com/json-widgets/bolts-factory-ptcs.json',
				'import_customizer_file_url' => 'http://artifacts.proteusthemes.com/customizer-exports/bolts-factory.dat',
				'import_preview_image_url'   => 'http://artifacts.proteusthemes.com/import-preview-images/bolts-factory.jpg',
				'import_notice'              => sprintf( esc_html__( 'Bolts Factory demo uses Slider Revolution plugin. You will also have to install and activate the Slider Revolution plugin and separately import the  demo slider. %1$sMore info here%2$s.', 'bolts-pt' ), '<a href="https://www.proteusthemes.com/docs/bolts/#factory-slider-revolution-import" target="_blank">', '</a>' ),
				'preview_url'                => 'https://demo.proteusthemes.com/bolts-factory',
			),
			array(
				'import_file_name'           => 'Bolts Industry',
				'import_file_url'            => 'http://artifacts.proteusthemes.com/xml-exports/bolts-industry-latest.xml',
				'import_widget_file_url'     => 'http://artifacts.proteusthemes.com/json-widgets/bolts-industry-ptcs.json',
				'import_customizer_file_url' => 'http://artifacts.proteusthemes.com/customizer-exports/bolts-industry.dat',
				'import_preview_image_url'   => 'http://artifacts.proteusthemes.com/import-preview-images/bolts-industry.jpg',
				'preview_url'                => 'https://demo.proteusthemes.com/bolts-industry',
			),
		);
	}


	/**
	 * After import theme setup for One Click Demo Import plugin.
	 */
	function ocdi_after_import_setup( $selected_import ) {

		// Menus to Import and assign.
		$main_menu = get_term_by( 'name', 'Main Menu', 'nav_menu' );

		set_theme_mod( 'nav_menu_locations', array(
				'main-menu'   => $main_menu->term_id,
			)
		);

		// Set options for front page and blog page.
		$front_page_id = get_page_by_title( 'Home' );
		$blog_page_id  = get_page_by_title( 'News' );

		update_option( 'show_on_front', 'page' );
		update_option( 'page_on_front', $front_page_id->ID );
		update_option( 'page_for_posts', $blog_page_id->ID );

		// Set options for Breadcrumbs NavXT
		$breadcrumbs_settings = get_option( 'bcn_options', array() );
		$breadcrumbs_settings['hseparator'] = '';
		$shop_page = get_page_by_title( 'Shop' );
		if ( ! is_null( $shop_page ) ) {
			$breadcrumbs_settings['apost_product_root'] = $shop_page->ID;
		}
		$breadcrumbs_settings['bpost_product_archive_display'] = false;
		$projects_page = null;
		if ( 'Bolts Transport' === $selected_import['import_file_name'] ) {
			$projects_page = get_page_by_title( 'Our Vehicles' );
		}
		else {
			$projects_page = get_page_by_title( 'Our Projects' );
		}
		if ( ! is_null( $projects_page ) ) {
			$breadcrumbs_settings['apost_portfolio_root'] = $projects_page->ID;
		}
		$breadcrumbs_settings['bpost_portfolio_archive_display'] = false;
		$breadcrumbs_settings['bpost_portfolio_taxonomy_display'] = false;
		update_option( 'bcn_options', $breadcrumbs_settings );

		esc_html_e( 'After import setup ended!', 'bolts-pt' );
	}


	/**
	 * Message for manual demo import for One Click Demo Import plugin.
	 */
	function ocdi_message_after_file_fetching_error() {
		return sprintf( esc_html__( 'Please try to manually import the demo data. Here are instructions on how to do that: %sDocumentation: Import XML File%s', 'bolts-pt' ), '<a href="https://www.proteusthemes.com/docs/bolts/#import-xml-file" target="_blank">', '</a>' );
	}


	/**
	 * Append right body classes to the body
	 *
	 * @return string
	 */
	public function body_classes( $classes ) {
		if ( 'boxed' === get_theme_mod( 'layout_mode', 'wide' ) ) {
			$classes[] = 'boxed';
		}

		$classes = BoltsHelpers::get_fonts_classes( $classes );

		return $classes;
	}


	/**
	 * Add PW widgets to Page Builder group and add icon class.
	 *
	 * @param array $widgets All widgets in page builder list of widgets.
	 *
	 * @return array
	 */
	function add_icons_to_page_builder_for_pw_widgets( $widgets ) {
		foreach ( $widgets as $class => $widget ) {
			if ( strstr( $widget['title'], 'ProteusThemes:' ) ) {
				$widgets[ $class ]['icon']   = 'pw-pb-widget-icon';
				$widgets[ $class ]['groups'] = array( 'pw-widgets' );
			}
		}

		return $widgets;
	}


	/**
	 * Add another tab section in the Page Builder "add new widget" dialog.
	 *
	 * @param array $tabs Existing tabs.
	 *
	 * @return array
	 */
	function siteorigin_panels_add_widgets_dialog_tabs( $tabs ) {
		$tabs['pw_widgets'] = array(
			'title' => esc_html__( 'ProteusThemes Widgets', 'bolts-pt' ),
			'filter' => array(
				'groups' => array( 'pw-widgets' ),
			),
		);

		return $tabs;
	}


	/**
	 * Set to which customizer panel the sticky menu should be appended.
	 *
	 * @param array $defaults Default settings for this filter.
	 *
	 * @return array
	 */
	function pt_sticky_menu_theme_panel( $defaults ) {
		$defaults['panel'] = 'panel_bolts';

		return $defaults;
	}


	/**
	 * Set the mobile menu button class for the sticky menu.
	 */
	function pt_sticky_menu_mobile_menu_button_class() {
		return 'btn-dark';
	}
}

// Single instance.
$bolts_filters = new BoltsFilters();
