<?php
/**
 * Register sidebars for Bolts
 *
 * @package bolts-pt
 */

function bolts_sidebars() {
	// Blog Sidebar.
	register_sidebar(
		array(
			'name'          => esc_html__( 'Blog Sidebar', 'bolts-pt' ),
			'id'            => 'blog-sidebar',
			'description'   => esc_html__( 'Sidebar on the blog layout.', 'bolts-pt' ),
			'class'         => 'blog  sidebar',
			'before_widget' => '<div class="widget  %2$s">',
			'after_widget'  => '</div>',
			'before_title'  => '<h4 class="sidebar__headings">',
			'after_title'   => '</h4>',
		)
	);

	// Regular Page Sidebar.
	register_sidebar(
		array(
			'name'          => esc_html__( 'Regular Page Sidebar', 'bolts-pt' ),
			'id'            => 'regular-page-sidebar',
			'description'   => esc_html__( 'Sidebar on the regular page.', 'bolts-pt' ),
			'class'         => 'sidebar',
			'before_widget' => '<div class="widget  %2$s">',
			'after_widget'  => '</div>',
			'before_title'  => '<h4 class="sidebar__headings">',
			'after_title'   => '</h4>',
		)
	);

	// Register top sidebars only if the setting 'top_bar_visibility' in customizer is set to be visible.
	if ( 'no' !== get_theme_mod( 'top_bar_visibility', 'yes' )  ) {
		// Top Left Widgets.
		register_sidebar(
			array(
				'name'          => esc_html__( 'Top Left', 'bolts-pt' ),
				'id'            => 'top-left-widgets',
				'description'   => esc_html__( 'Top left widget area for Icon Box, Social Icons, Custom Menu or Text widget.', 'bolts-pt' ),
				'before_widget' => '<div class="widget  %2$s">',
				'after_widget'  => '</div>',
			)
		);

		// Top Right Widgets.
		register_sidebar(
			array(
				'name'          => esc_html__( 'Top Right', 'bolts-pt' ),
				'id'            => 'top-right-widgets',
				'description'   => esc_html__( 'Top right widget area for Icon Box, Social Icons, Custom Menu or Text widget.', 'bolts-pt' ),
				'before_widget' => '<div class="widget  %2$s">',
				'after_widget'  => '</div>',
			)
		);
	}

	// Header Widgets.
	register_sidebar(
		array(
			'name'          => esc_html__( 'Header', 'bolts-pt' ),
			'id'            => 'header-widgets',
			'description'   => esc_html__( 'Header widget area for Icon Box, Buttons (Textwidget), Social Icons, Search, or Skype Button.', 'bolts-pt' ),
			'before_widget' => '<div class="widget  %2$s">',
			'after_widget'  => '</div>',
		)
	);

	// Show this sidebar only when Kraft header is used.
	if ( 'kraft' === get_theme_mod( 'theme_header', 'kraft' ) ) {
		// Navigation Widgets.
		register_sidebar(
			array(
				'name'          => esc_html__( 'Navigation', 'bolts-pt' ),
				'id'            => 'navigation-widgets',
				'description'   => esc_html__( 'Navigation widget area for Buttons (Textwidget), Social Icons or Search.', 'bolts-pt' ),
				'before_widget' => '<div class="widget  %2$s">',
				'after_widget'  => '</div>',
			)
		);
	}

	// WooCommerce shop sidebar.
	if ( BoltsHelpers::is_woocommerce_active() ) {
		register_sidebar(
			array(
				'name'          => esc_html__( 'Shop Sidebar', 'bolts-pt' ),
				'id'            => 'shop-sidebar',
				'description'   => esc_html__( 'Sidebar for the shop page', 'bolts-pt' ),
				'class'         => 'sidebar',
				'before_widget' => '<div class="widget  %2$s">',
				'after_widget'  => '</div>',
				'before_title'  => '<h4 class="sidebar__headings">',
				'after_title'   => '</h4>',
			)
		);
	}

	// Footer.
	$footer_widgets_num = count( BoltsHelpers::footer_widgets_layout_array() );

	// Only register if not 0.
	if ( $footer_widgets_num > 0 ) {
		register_sidebar(
			array(
				'name'          => esc_html__( 'Footer', 'bolts-pt' ),
				'id'            => 'footer-widgets',
				'description'   => sprintf( esc_html__( 'Footer area works best with %d widgets. This number can be changed in the Appearance &rarr; Customize &rarr; Theme Options &rarr; Footer.', 'bolts-pt' ), $footer_widgets_num ),
				'before_widget' => '<div class="col-xs-12  col-lg-__col-num__"><div class="widget  %2$s">', // __col-num__ is replaced dynamically in filter 'dynamic_sidebar_params'
				'after_widget'  => '</div></div>',
				'before_title'  => '<h6 class="footer-top__headings">',
				'after_title'   => '</h6>',
			)
		);
	}
}
add_action( 'widgets_init', 'bolts_sidebars' );
