<?php
/**
 * Single portfolio page template
 *
 * @package bolts-pt
 */

get_header();

get_template_part( 'template-parts/page-header' );

?>

	<div id="primary" class="content-area  container">
		<main>
			<div class="row">
				<div class="col-xs-12  col-lg-4">

					<?php
					while ( have_posts() ) :
						the_post();
					?>
						<article id="post-<?php the_ID(); ?>" <?php post_class( 'portfolio__content' ); ?>>
							<h2 class="portfolio__title"><?php echo get_the_title(); ?></h2>
							<?php if ( ( $price = get_field( 'price' ) ) && get_field( 'show_price' ) ) : ?>
								<p class="portfolio__price"><?php echo esc_html( $price ); ?></p>
							<?php endif; ?>
							<?php if ( $label = get_field( 'label' ) ) : ?>
								<div class="portfolio__label"><?php echo esc_html( $label ); ?></div>
							<?php endif; ?>

							<?php
							// Holds data for specifications repeating fields.
							$specifications = array();

							// Add additional fields.
							while ( have_rows( 'specification' ) ) {
								the_row();
								$specifications[] = array(
									'icon'    => get_sub_field( 'icon' ),
									'text'    => get_sub_field( 'text' ),
									'display' => get_sub_field( 'display' ),
								);
							}
							?>
							<div class="portfolio__specification">
								<?php foreach ( $specifications as $row ) : ?>
									<div class="portfolio__specification-item">
										<i class="fa  <?php echo esc_attr( $row['icon'] ); ?>"></i>
										<?php echo esc_html( $row['text'] ); ?>
									</div>
								<?php endforeach; ?>
							</div>

							<div class="hentry__content">
								<?php the_content(); ?>
							</div>
						</article>
					<?php
						endwhile;
					?>

					<!-- Portfolio Navigation -->
					<nav class="portfolio__navigation">
						<?php
							BoltsHelpers::get_next_prev_portfolio_link( true, esc_html__( 'Previous', 'bolts-pt' ) );
							BoltsHelpers::get_next_prev_portfolio_link( false, esc_html__( 'Next', 'bolts-pt' ) );
						?>
					</nav>
				</div>

				<?php
					// Gallery or editor?
					$project_right = get_field( 'project_right' );
				?>

				<div class="col-xs-12  col-lg-8"<?php echo ( 'gallery' === $project_right ) ? ' data-featherlight-gallery data-featherlight-filter="a"' : ''; ?>>
					<?php if ( has_post_thumbnail() ) : ?>
						<a href="<?php echo esc_url( wp_get_attachment_url( get_post_thumbnail_id() ) ); ?>">
							<?php the_post_thumbnail( 'post-thumbnail', array( 'class' => 'img-fluid  hentry__featured-image' ) ); ?>
						</a>
					<?php endif; ?>

					<?php if ( 'editor' === $project_right ) : ?>
						<div class="portfolio__right-content">
							<?php the_field( 'project_content' ); ?>
						</div>
					<?php else : ?>
						<?php $portfolio_gallery_columns = get_theme_mod( 'portfolio_gallery_columns', 2 ); ?>
						<div class="portfolio__gallery  portfolio__gallery--col-<?php echo absint( $portfolio_gallery_columns ); ?>">
						<?php
						while ( have_rows( 'project_gallery' ) ) {
							the_row();

							$image = get_sub_field( 'project_image' );

							printf(
								'<a class="portfolio__gallery-link" href="%1$s"><img class="img-fluid  portfolio__gallery-image" src="%2$s" alt="%3$s" width="%4$d" height="%5$d" /></a> ',
								esc_url( $image['url'] ),
								esc_attr( $image['sizes']['bolts-portfolio-gallery'] ),
								esc_attr( $image['title'] ),
								absint( $image['sizes']['bolts-portfolio-gallery-width'] ),
								absint( $image['sizes']['bolts-portfolio-gallery-height'] )
							);
						}
						?>
						</div>
					<?php endif; ?>
				</div>
			</div>
		</main>
	</div>

<?php get_footer(); ?>
