<?php
/**
 * The page title part of the header
 *
 * @package bolts-pt
 */

$bolts_style_attr = '';

// Regular page id.
$bolts_bg_id        = get_the_ID();
$bolts_blog_id      = absint( get_option( 'page_for_posts' ) );
$bolts_shop_id      = absint( get_option( 'woocommerce_shop_page_id', 0 ) );
$bolts_portfolio_id = absint( get_theme_mod( 'portfolio_parent_page', 0 ) );

// If blog or single post use the ID of the blog.
if ( is_home() || is_singular( 'post' ) ) {
	$bolts_bg_id = $bolts_blog_id;
}

// If woocommerce page, use the shop page id.
if ( BoltsHelpers::is_woocommerce_active() && is_woocommerce() ) {
	$bolts_bg_id = $bolts_shop_id;
}

// If Portfolio widget is activate use the parent page for single portfolio pages.
if ( BoltsHelpers::is_portfolio_plugin_active() && is_singular( 'portfolio' ) ) {
	$bolts_bg_id = $bolts_portfolio_id;
}

$show_title_area = get_field( 'show_title_area', $bolts_bg_id );
if ( ! $show_title_area ) {
	$show_title_area = 'yes';
}

$show_breadcrumbs = get_field( 'show_breadcrumbs', $bolts_bg_id );
if ( ! $show_breadcrumbs ) {
	$show_breadcrumbs = 'yes';
}

// Show/hide page title area (ACF control - single page && customizer control - all pages).
if ( 'yes' === $show_title_area && 'yes' === get_theme_mod( 'show_page_title_area', 'yes' ) ) :

	$bolts_style_attr = BoltsHelpers::page_header_background_style( $bolts_bg_id );

	?>

	<div class="page-header"<?php echo empty( $bolts_style_attr ) ? '' : ' style="' . esc_attr( $bolts_style_attr ) . ';"'; ?>>
		<div class="container">
			<div class="page-header__text">
				<?php
				$bolts_main_tag = 'h1';
				$bolts_subtitle = false;

				if ( is_home() || ( is_single() && 'post' === get_post_type() ) ) {
					$bolts_title    = 0 === $bolts_blog_id ? esc_html__( 'Blog', 'bolts-pt' ) : get_the_title( $bolts_blog_id );
					$bolts_subtitle = get_field( 'subtitle', $bolts_blog_id );

					if ( is_single() ) {
						$bolts_main_tag = 'h2';
					}
				}
				elseif ( BoltsHelpers::is_woocommerce_active() && is_woocommerce() ) {
					ob_start();
					woocommerce_page_title();
					$bolts_title    = ob_get_clean();
					$bolts_subtitle = get_field( 'subtitle', $bolts_shop_id );

					if ( is_product() ) {
						$bolts_main_tag = 'h2';
					}
				}
				elseif ( BoltsHelpers::is_portfolio_plugin_active() && is_singular( 'portfolio' ) ) {
					$bolts_title    = get_the_title( $bolts_portfolio_id );
					$bolts_subtitle = get_field( 'subtitle', $bolts_portfolio_id );
				}
				elseif ( is_category() || is_tag() || is_author() || is_post_type_archive() || is_tax() || is_day() || is_month() || is_year() ) {
					$bolts_title = get_the_archive_title();
				}
				elseif ( is_search() ) {
					$bolts_title = esc_html__( 'Search Results For' , 'bolts-pt' ) . ' &quot;' . get_search_query() . '&quot;';
				}
				elseif ( is_404() ) {
					$bolts_title = esc_html__( 'Error 404' , 'bolts-pt' );
				}
				else {
					$bolts_title    = get_the_title();
					$bolts_subtitle = get_field( 'subtitle' );
				}

				?>

				<?php printf( '<%1$s class="page-header__title">%2$s</%1$s>', tag_escape( $bolts_main_tag ), esc_html( $bolts_title ) ); ?>

				<?php if ( $bolts_subtitle ) : ?>
					<p class="page-header__subtitle"><?php echo esc_html( $bolts_subtitle ); ?></p>
				<?php endif; ?>
			</div>

			<?php
			if ( 'yes' === $show_breadcrumbs ) {
				get_template_part( 'template-parts/breadcrumbs' );
			}
			?>

		</div>
	</div>
<?php endif; ?>
